/*
 * emma.h - generic routines
 */

#ifndef _EMMA_H_
#define _EMMA_H_

#ifdef CONFIG_SOC_EMMA3SLx
#include <asm/arch-emma/emma3slp.h>
#endif

#ifdef CONFIG_SOC_EMMA3SEP
#include <asm/arch-emma/emma3sep.h>
#endif

#ifndef __ASSEMBLY__

void static inline __delay(unsigned long loops)
{
	if (sizeof(long) == 4)
		__asm__ __volatile__ (
			".set\tnoreorder\n"
			"1:\tbnez\t%0,1b\n\t"
			"subu\t%0,1\n\t"
			".set\treorder"
			: "=r" (loops)
			: "0" (loops));
	else if (sizeof(long) == 8)
		__asm__ __volatile__ (
			".set\tnoreorder\n"
			"1:\tbnez\t%0,1b\n\t"
			"dsubu\t%0,1\n\t"
			".set\treorder"
			: "=r" (loops)
			: "0" (loops));
}

/* cpu pipeline flush */
void static inline emma_sync(void)
{
	__asm__ volatile ("sync");
}

void static inline emma_sync_udelay(int us)
{
	__asm__ volatile ("sync");
	udelay(us);
}

void static inline emma_writeb(u8 val, int reg)
{
	*(volatile u8 *)(reg) = val;
}

void static inline emma_writew(u16 val, int reg)
{
	*(volatile u16 *)(reg) = val;
}

void static inline emma_writel(u32 val, int reg)
{
	*(volatile u32 *)(reg) = val;
}

static inline u8 emma_readb(unsigned long port)
{
	return *(volatile u8 *)port;
}

static inline u16 emma_readw(unsigned long port)
{
	return *(volatile u16 *)port;
}

static inline u32 emma_readl(unsigned long port)
{
	return *(volatile u32 *)port;
}

void static inline emma_resetw(u32 val1, unsigned delay, u32 val2, volatile u32 *reg)
{
	*(volatile u32 *)(reg) = val1;
	emma_sync_udelay(delay);
	*(volatile u32 *)(reg) = val2;
}

#endif /* !ASSEMBLY */

#ifdef CONFIG_PM
/* no CP0 timer irq */
#define ALLINTS (IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4)
#else
#define ALLINTS (IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5)
#endif

#endif /* _EMMA_H_ */
