/*
 * Memory sub-system initialization code for EMMA3SE/P development board.
 *
 * Copyright (C) 2011 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>

/**********************************************************************/
#define WAIT_100NS		(40)

/* Configuration the DDR macro parameter */
#ifdef CONFIG_EMMA_DDR3
#define DLIF_FUNCCTRL_DATA	0x00000101	/* FUNCRSTB = 1, VDD = 1.5V */
#define DLIF_DLLCTRL_DATA0	0x00000001	/* MDLLSTBY = 1, RCLK = 640-800MHz */
#define DLIF_DLLCTRL_DATA1	0x00000000	/* MDLLSTBY = 0, RCLK = 640-800MHz */
#define DLIF_ZQCALCTRL_DATA0	0x00000182	/* ZQCALRSTB = 0, ZQCALITVL = 8 */
						/* RCLK = 400-800MHz, DFICLK = 200-400MHz */
#define DLIF_ZQCALCTRL_DATA1	0x00000183	/* ZQCALRSTB = 1, ZQCALITVL = 8 */
						/* RCLK = 400-800MHz, DFICLK = 200-400MHz */
#define DLIF_ZQODTCTRL_DATA	0xeaab8051	/* DDR3-1333 */
#define DLIF_RDCTRL_DATA	0xd646d645	/* DDR3-1333 */
#define DLIF_RDTMG_DATA		0x000000b0	/* DDR3-1333 */
#define DLIF_FIFOINIT_DATA	0x00000101	/* RDPINIT = 1, WDPINIT = 1 */
#define DLIF_OUTCTRL_DATA	0x02110f07	/* WL=15, RL=17, BL=8, All Output Enable */
#define DLIF_WLCTRL1_DATA	0x00000000
#define DLIF_DQCALOFS_DATA	0x00004646
#define DLIF_DQCALEXP_DATA	0x800000aa

#define MIF_REFCNT_DATA		0x0000019c	/* 324MHz */
#define MIF_ZQCCNT_DATA		0x000001ff

#else	/* DDR2 SDRAM */
#define DLIF_FUNCCTRL_DATA	0x00000001	/* FUNCRSTB = 1, VDD = 1.8V */
#define DLIF_DLLCTRL_DATA0	0x00000007	/* MDLLSTBY = 1, RCLK = 400-460MHz */
#define DLIF_DLLCTRL_DATA1	0x00000006	/* MDLLSTBY = 0, RCLK = 400-460MHz */
#define DLIF_ZQCALCTRL_DATA0	0x00000186	/* ZQCALRSTB = 0, ZQCALITVL = 8 */
						/* RCLK = 333-667MHz, DFICLK = 100-200MHz */
#define DLIF_ZQCALCTRL_DATA1	0x00000187	/* ZQCALRSTB = 1, ZQCALITVL = 8 */
						/* RCLK = 333-667MHz, DFICLK = 100-200MHz */
#define DLIF_ZQODTCTRL_DATA	0xab330050	/* DDR2-800, No FIFO */
#define DLIF_RDCTRL_DATA	0x94449443	/* DDR2-800 */
#define DLIF_RDTMG_DATA		0x000000a0	/* DDR2-800 */
#define DLIF_FIFOINIT_DATA	0x00000101	/* RDPINIT = 1, WDPINIT = 1 */
#define DLIF_OUTCTRL_DATA	0x01090803	/* WL=8, RL=9, BL=4, All Output Enable */
#define DLIF_WLCTRL1_DATA	0x00000000
#define DLIF_DQCALOFS_DATA	0x00004646
#define DLIF_DQCALEXP_DATA	0x800000aa

#define MIF_REFCNT_DATA		0x00000160	/* 200MHz */
#define MIF_ZQCCNT_DATA		0x000000ff	/* default */
#endif

/*
 * PORT, ROM I/F Init DATA
 */
/* T.B.D */
#define HIF_SCPU_PADR0_MASK	0x0003FFFE
#define HIF_SCPU_PADR0_TARGET	0x00000002	/* 0x00002000 -> 0x00000000, 8KB */
#define HIF_SCPU_PADR1_BASE	0x00000000
#define HIF_SCPU_PADR1_MASK	0x00000000
#define HIF_SCPU_PADR1_TARGET	0x00000000
#define HIF_SCPU_PADR2_BASE	0x00000000
#define HIF_SCPU_PADR2_MASK	0x00000000
#define HIF_SCPU_PADR2_TARGET	0x00000000
#define HIF_SCPU_PADR_EN_WIN	0x00000001	/* Win0 */

#if defined(CONFIG_NAND_BOOT)
#define ROM_MODE_DATA		0x000e0811	/* 8MB, CS0=NAND, CS1=Serial */
#define ROM_IF_DATA		0x01000003
#elif defined(CONFIG_SERIAL_BOOT)
#define ROM_MODE_DATA		0x00030000	/* 4MB, CS0=Serial CS1=none */
#define ROM_IF_DATA		0x00000001
#else
#define ROM_MODE_DATA		0x00000001	/* 8MB, CS0=NOR, CS1=none */
#define ROM_IF_DATA		0x00000001
#endif
#define ROM_ACCESS_TIME_DATA	0x00000000

	.text
	.set noreorder
	.set mips32

	.globl	lowlevel_init
lowlevel_init:

	.macro	REGWR base, offset, val
	li	t1, \val
	sw	t1, \offset(\base)
	.endm

	.macro	MEM_WAIT val
	li	t2, \val
	1:
	bne	t2, zero, 1b
	addi	t2, -1
	.endm

	/* PBRD Init. */
	li	t0, KSEG1ADDR(PBRD_BASE)
	REGWR	t0, PBRD_INTENABLE_MAIN_OFS, 0xc0		/* uart interrupt enable */

	/*
	 * ROM I/F Init.
	 */
	li	t0, KSEG1ADDR(BROI_BASE)
	REGWR	t0, BROI_ROM_MODE_OFS,	      ROM_MODE_DATA
/*	REGWR	t0, BROI_ROM_ACCESS_MODE_OFS, ROM_ACCESS_MODE_DATA */
	lw	t1, BROI_ROMIF_MODE_OFS(t0)
	li	t2, ROM_IF_DATA
	or	t1, t1, t2
	sw	t1, BROI_ROMIF_MODE_OFS(t0)

PORT_INIT_END:

	/*
	  checks whether the fetch address is RAM(0x8XXXXXXX,0xAXXXXXXX)
	  or ROM(0x9XXXXXXX,0xBXXXXXXX).
	  If the address is RAM, memory initialization should skip.
	*/
	li	t0, 0x10000000
	and	t1, t0, ra
	xor	t1, t1, t0
	bnez	t1, MEM_INIT_END
	nop

MEM_INIT_START:
	/*
	 * Configure EMMA DDR memory-bus
	 */

	li	t0, KSEG1ADDR(MIF_BASE)
	/*
	 * DDR3 macro Initialize
	*/
	/* External Pin MRSTCNT=0, MCKECNT=0 */
	REGWR	t0, MIF_DLIF_RSTB_OFS, 0x00000001	/* PRESETn = 1 */
	REGWR	t0, MIF_RSTB_OFS,      0x00000001	/* URSTB = 1 */

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000000	/* FUNCCTRL */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_FUNCCTRL_DATA
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000100	/* DLLCTRL */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_DLLCTRL_DATA0
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_DLLCTRL_DATA1
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000200	/* ZQCALCTRL */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_ZQCALCTRL_DATA0
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_ZQCALCTRL_DATA1
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000300	/* ZQODTCTRL */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_ZQODTCTRL_DATA
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000400	/* RDCTRL */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_RDCTRL_DATA
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000500	/* RDTMG */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_RDTMG_DATA
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000600	/* FIFOINIT */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_FIFOINIT_DATA
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_CLKDISABLE_OFS,0x00000000	/* CLKDISABLE */
	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00000700	/* OUTCTRL */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_OUTCTRL_DATA
	MEM_WAIT (2 * WAIT_100NS)

#ifdef CONFIG_EMMA_DDR3
	/* External Pin MRSTCNT=1, MCKECNT=1 */
	REGWR	t0, MIF_DDRRSTB_OFS,   0x00000001	/* DDR3 RESETB = 1 */
	MEM_WAIT (5000 * WAIT_100NS)
#endif

	/*
	 * Set MIF CONFIG
	*/
#ifdef CONFIG_EMMA_DDR3
	/* 2Gb(x16) CL9 16-bit bus  FlighTime=0-3T */
	REGWR	t0, MIF_CONFIG_OFS,  0xa74e9f9a
	REGWR	t0, MIF_CONFIG2_OFS, 0x4210fec0
	REGWR	t0, MIF_CONFIG3_OFS, 0x000a000a

	/*
	 * SDRAM initialization
	*/
	REGWR	t0, MIF_INIT_OFS, 0x00000020		/* CKE=1 */
	MEM_WAIT (2 * WAIT_100NS)

	REGWR	t0, MIF_INIT_OFS, 0x02102002		/* EMR2 CWL=7 */
	REGWR	t0, MIF_INIT_OFS, 0x00003002		/* EMR3 */
	REGWR	t0, MIF_INIT_OFS, 0x000c1002		/* EMR1 DDL enable, AL=CL-1 */
	REGWR	t0, MIF_INIT_OFS, 0x1b500002		/* MR0 DLL reset, CL=9, WR=10 */
	REGWR	t0, MIF_INIT_OFS, 0x04000040		/* ZQCL */
	MEM_WAIT (3 * WAIT_100NS)			/* Wait for 256 MCLK */
#else	/* DDR2_SDRAM */
	/* 1Gb(x16) CL5 16-bit bus  FlighTime=0-3T */
	REGWR	t0, MIF_CONFIG_OFS,  0x94e71696
	REGWR	t0, MIF_CONFIG2_OFS, 0x12084280
	REGWR	t0, MIF_CONFIG3_OFS, 0x01090006

	/*
	 * SDRAM initialization
	*/
	REGWR	t0, MIF_INIT_OFS, 0x00000020		/* CKE=1 */
	MEM_WAIT (2 * WAIT_100NS)

	REGWR	t0, MIF_INIT_OFS, 0x04000004		/* All Banks Precharge */
	REGWR	t0, MIF_INIT_OFS, 0x00002002		/* EMR2 */
	REGWR	t0, MIF_INIT_OFS, 0x00003002		/* EMR3 */
	REGWR	t0, MIF_INIT_OFS, 0x00001002		/* EMR1 DLL enabele, AL=0 */
	REGWR	t0, MIF_INIT_OFS, 0x0b520002		/* MR0 DLL reset, CL=5, TWR=6 */
	MEM_WAIT (1 * WAIT_100NS)			/* Wait for 100 MCLK */

	REGWR	t0, MIF_INIT_OFS, 0x04000004		/* All Banks Precharge */
	REGWR	t0, MIF_INIT_OFS, 0x00000001		/* Refresh */
	MEM_WAIT (2 * WAIT_100NS)

	REGWR	t0, MIF_INIT_OFS, 0x00000001		/* Refresh */
	MEM_WAIT (2 * WAIT_100NS)

	REGWR	t0, MIF_INIT_OFS, 0x0a520002		/* MR0 CL=5, TWR=6 */
	REGWR	t0, MIF_INIT_OFS, 0x03801002		/* EMR1 OCD default, AL=0 */
	REGWR	t0, MIF_INIT_OFS, 0x00241002		/* EMR1 OCD exit, AL=4, ODT=75orm, Drive=Full */
#endif

	/*
	 * Write Leveling
	*/
	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00001000	/* WLCTRL1 */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_WLCTRL1_DATA

	/*
	 * DQ Calibration
	*/
	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00003a00	/* DQCALOFS */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_DQCALOFS_DATA
	MEM_WAIT (3 * WAIT_100NS)

	REGWR	t0, MIF_DLIF_CTRL_OFS, 0x00003e00	/* DQCALEXP */
	REGWR	t0, MIF_DLIF_DATA_OFS, DLIF_DQCALEXP_DATA

	/*
	 * Set Refresh Counter
	*/
	REGWR	t0, MIF_REFCNT_OFS, MIF_REFCNT_DATA

	/*
	 * Set ZQ Calibration counter
	*/
	REGWR	t0, MIF_ZQCCNT_OFS, MIF_ZQCCNT_DATA

	/*
	 * Set up internal arbitartion for SDRAM
	*/
	li	t0, KSEG1ADDR(MIF_BASE + 0x0300)

#ifdef CONFIG_EMMA_DDR3
	REGWR	t0, 0x14, 0x00804020
	REGWR	t0, 0x20, 0x50804020
	REGWR	t0, 0x24, 0x50804020
	REGWR	t0, 0x28, 0x50804020
	REGWR	t0, 0x2c, 0x48804020
	REGWR	t0, 0x30, 0x00fce0c0
	REGWR	t0, 0x34, 0x00fce0c0
	REGWR	t0, 0x40, 0x0c301000
	REGWR	t0, 0x44, 0x0c301000
	REGWR	t0, 0x48, 0x0c301000
	REGWR	t0, 0x4c, 0x00c07050
	REGWR	t0, 0x58, 0x00100000
	REGWR	t0, 0x60, 0x00fce0e0
	REGWR	t0, 0x70, 0x00c06020
	REGWR	t0, 0x78, 0x00804010
	REGWR	t0, 0x7c, 0x00502000
#else	/* DDR2 */
	REGWR	t0, 0x14, 0x00603010
	REGWR	t0, 0x20, 0xfcfce0c0
	REGWR	t0, 0x24, 0xfcfce0c0
	REGWR	t0, 0x28, 0xfcfce0c0
	REGWR	t0, 0x2c, 0xfcfce0c0
	REGWR	t0, 0x30, 0x00fce0c0
	REGWR	t0, 0x34, 0x00fce0c0
	REGWR	t0, 0x40, 0x0c301000
	REGWR	t0, 0x44, 0x0c301000
	REGWR	t0, 0x48, 0x0c301000
	REGWR	t0, 0x4c, 0x30fcc0a0
	REGWR	t0, 0x58, 0x00100000
	REGWR	t0, 0x60, 0x00fce0e0
	REGWR	t0, 0x70, 0x00c06020
	REGWR	t0, 0x78, 0x00804010
	REGWR	t0, 0x7c, 0x00502000
#endif

MEM_INIT_END:

WIN_INIT_START:
	/* Memory Window setting for Sub CPU */
	li	t0, KSEG1ADDR(BHIF_BASE)
	li	t2, HIF_SCPU_PADR_EN_WIN
	li	t1, 0x00000001
	and	t1, t1, t2
	blez	t1, swin1
	nop
swin0:
	REGWR	t0, BHIF_SUB_PADR0_MASK_OFS,   HIF_SCPU_PADR0_MASK
	REGWR	t0, BHIF_SUB_PADR0_TARGET_OFS, HIF_SCPU_PADR0_TARGET
swin1:
	li	t1, 0x00000100
	and	t1, t1, t2
	blez	t1, swin2
	nop
	REGWR	t0, BHIF_SUB_PADR1_BASE_OFS,   HIF_SCPU_PADR1_BASE
	REGWR	t0, BHIF_SUB_PADR1_MASK_OFS,   HIF_SCPU_PADR1_MASK
	REGWR	t0, BHIF_SUB_PADR1_TARGET_OFS, HIF_SCPU_PADR1_TARGET
swin2:
	li	t1, 0x00010000
	and	t1, t1, t2
	blez	t1, swin_en
	nop
	REGWR	t0, BHIF_SUB_PADR2_BASE_OFS,   HIF_SCPU_PADR2_BASE
	REGWR	t0, BHIF_SUB_PADR2_MASK_OFS,   HIF_SCPU_PADR2_MASK
	REGWR	t0, BHIF_SUB_PADR2_TARGET_OFS, HIF_SCPU_PADR2_TARGET

swin_en:
	REGWR	t0, BHIF_SUB_PADR_EN_OFS,      HIF_SCPU_PADR_EN_WIN	/* Enable Sub CPU Window */

WIN_INIT_END:

	jr	ra
	nop

