/*
 * GPIOの初期設定 for Pixela製AitTunerRec基板 by H.HASEGAWA @ 2012.03.01 refs #18 #25
 * (\\Daisan\daisan\Projects\POE-BR320W\Specification\HW仕様書\
 *  POE-BR320】EMMA3SE部_HW仕様0.2_120215.pdf を参照)
 *
 * K.UEDA
 * 2013.05.10
 *
 * H.HASEGAWA
 * 2012.03.31
 *
 */

#define RST_ETHPHY_GPIO_NUM	13

static void _setup_gpio(void)
{
	unsigned long avsw, alt, dat, dir;
	
	/* ------------
	 * GPIO初期設定
	 * ------------ */
	dat = dir = 0;
	// GPIO[ 0~ 1] : (I) (シェアピンで 他機能(TS SideA 入力)として使用)
	;
	// GPIO[ 2~ 8] : (O) (未使用)=L x 7
	dir |= 0x7fLU << 2;
	// GPIO[ 9:11] : (I) (シェアピンで 他機能(TS SideA 入力)として使用)
	;
	// GPIO[   12] : (O) TU_RST#=L
	dir |= 0x01LU << 12;
	// GPIO[   13] : (O) RST_ETHPHY=L
	dir |= 0x01LU << RST_ETHPHY_GPIO_NUM;
	// GPIO[14~16] : (O) PERRST=L, CAPTURE_POWER=L(Dione電源オン), H57FW_WEX=H
	//                      DioneTSパラレル入力ピンの何本かがプルアップされるので、Dione電源はオフしないほうがよい。
	dir |= 0x07LU << 14;
	dat |= 0x04LU << 14;
	// GPIO[17~18] : (I) NetRST, WPS_SW
	;
	// GPIO[19~24] : (O) LED1_A=H(Off), LED1_B=H(Off), LED2_A=H(Off), LED2_B=H(Off), USB1_ON=L(Off), USB_HUB_RST=L(Reset)
	//    LED点灯はgpio初期化ではなく、board_init_r()で行なう。 (refs #184)
	dir |= 0x3fLU << 19; // 
	dat |= 0x0fLU << 19;
	// GPIO[25~29] : (I) (シェアピンで 他機能(BCASカードI/F)として使用)
	;
	// GPIO[30~31] : (O) SELF_RST_EMMA_L=H, (未使用)=L
	dir |= 0x03LU << 30;
	dat |= 0x01LU << 30;
	
	emma_writel((~dir), ZPIO_MASK0); // 入力ピンのDATA値が書き変わらないようにMaskする！
	emma_writel(dat, ZPIO_DATA0);
	emma_writel(dir, ZPIO_DIR0);
	
	dat = dir = 0;
	// GPIO[   32] : (O) LNB_ON=L
	dir = 0x1LU << (32 - 32);
	// GPIO[33~34] : (I) LNB_MON, ET_INT
	;
	// GPIO[   35] : (O) (未使用)=L (シェアピンで 他機能(27Mhz CLKOUT)として将来使用予定だが 今は 未使用に)
	dir |= 0x1LU << (35 - 32);
	// GPIO[36~41] : (O) USB_POWER_ON_TAB_H=L(Power Off), USB_POWER_ON_HDD_H=L(Power Off), (未使用)=L x 4
	dir |= 0x3fLU << (36 - 32);
	dat |= 0x00LU << (36 - 32);
	// GPIO[42~46] : (I) (シェアピンで 他機能(TS SideB 入力)として使用)
	;
	// GPIO[47~50] : (O) USB_POWER_OFF_TAB=H, USB_POWER_OFF_HDD=H, BS1_ON_L=H, BS2_ON_L=H
	dir |= 0x0fLU << (47 - 32);
	dat |= 0x0fLU << (47 - 32);
	// GPIO[51~52] : (I) USB_OCR_ON_HDD_L, USB_OCR_ON_TAB_L
	;
	// GPIO[53~59] : (O) (未使用)=L x 7
	dir |= 0x7fLU << (53 - 32);
	
	emma_writel((~dir), ZPIO_MASK1); // 入力ピンのDATA値が書き変わらないようにMaskする！
	emma_writel(dat, ZPIO_DATA1);
	emma_writel(dir, ZPIO_DIR1);
	
	/* ---------------------
	 * PINファンクション設定
	 * --------------------- */
	/*
	 * データシート(R19DS0036ED0104_EMMA3SE_20111208.pdf)の 2.5.2 Shared Interface Conflicts 項の
	 * Table 19 PPORT (Programmable Pin Port) の表を見ながら、GPIOのPINファンクション設定を行う
	 * Table 19 中の "Option xx" の "xx" の番号は ALTレジスタ の bit位置 に対応しています
	 */
	alt = emma_readl(BHIF_ALTERNATE0);
	avsw = emma_readl(BHIF_AVSWITCH) & 0x0000000fLU;
	
	// GPIO[ 0~ 1] を TS SideA 入力    by ALT[19] = 1b
	// GPIO[ 2~ 5] を GPIO へ          by ALT[22,18] = 0b,0b
	// GPIO[ 6~ 8] を GPIO へ          by ALT[18] = 0b
	// GPIO[ 9~11] を TS SideA 入力    by ALT[19:18] = 10b
	alt  &= ~(0x01LU << 22);                       // ALT[22] = 0b
	alt  &= ~(0x03LU << 18); alt  |= 0x02LU << 18; // ALT[19:18] = 10b
	// EMMA内部の Stream Proc. の Serial in 0 へ TS0入力ピン(STS0)に入力されるTSデータを入力
	avsw &= ~(0x01LU);                             // AVSW[0] = 0b
	
	// GPIO[   12] は Not shared
	// GPIO[13~16] を GPIO へ          by ALT[21:20] = 00b
	// GPIO[   17] は Not shared
	// GPIO[18~19] を GPIO へ          by ALT[21] = 0b
	// GPIO[20~23] を GPIO へ          by ALT[21:20] = 00b
	// GPIO[   24] を GPIO へ          by ALT[21;20,0] = 00b,0b
	// GPIO[25~29] を BCASカードI/F へ by ALT[21:20,4,1:0] = 00b,1b,00b
	alt  &= ~(0x03LU << 20);                       // ALT[21:20] = 00b
	                         alt  |= 0x01LU <<  4; // ALT[4] = 1b
	alt  &= ~(0x03LU);                             // ALT[1:0] = 00b
	
	// GPIO[30~31] を GPIO へ          by ALT[5] = 0b
	// GPIO[47~48] を GPIO へ          by ALT[15,12] = 0b,0b
	// GPIO[49~51] を GPIO へ          by ALT[15,5] = 0b,0b
	// GPIO[   52] を GPIO へ          by ALT[15] = 0b
	alt  &= ~(0x01LU << 15);                       // ALT[15] = 0b
	alt  &= ~(0x01LU << 12);                       // ALT[12] = 0b
	alt  &= ~(0x01LU <<  5);                       // ALT[5] = 0b
	
	// GPIO[   32] は Not shared
	// GPIO[   33] を GPIO へ          by ALT[11] = 0b
	// GPIO[   34] を GPIO へ          by ALT[10] = 0b
	// GPIO[   35] を GPIO へ          by ALT[24,10] = 0b,0b
	// GPIO[36~37] を GPIO へ          by ALT[9] = 0b
	// GPIO[38~39] を GPIO へ          by ALT[7] = 0b
	// GPIO[40~41] を GPIO へ          by ALT[8] = 0b
	alt  &= ~(0x01LU << 24);                       // ALT[24] = 0b
	alt  &= ~(0x1fLU <<  7);                       // ALT[11:7] = 00000b
	
	// GPIO[42~46] を TS SideB 入力 へ by ALT[17] = 1b
	                         alt  |= 0x01LU << 17; // ALT[17] = 1b
	// EMMA内部の Stream Proc. の Serial in 1 へ TS1入力ピン(STS1)に入力されるTSデータを入力
	                         avsw |= 0x01LU <<  1; // AVSW[1] = 1b
	
	// GPIO[47~52] の設定は↑の GPIO[30~31] の設定を参照
	
	// GPIO[53~57] を GPIO へ          by ALT[26:25] = 00b
	// GPIO[58~59] は Not shared
	alt  &= ~(0x03LU << 25);                       // ALT[26:25] = 00b
	
	/* ------------------------------------------------------------
	 * TSパラレル出力PIN(RADD[15~25])からTS出力されるようにしておく
	 * ------------------------------------------------------------ */
	// TSパラレル出力PINからのTS出力は、MMAC_PLAYBACK と MMAC_DEMUX を利用して
	// Memory上のTSデータを出力します。
	// で、User Manual(R19UH0071ED0101_EMMA3SE_20120207.pdf)の44ページの
	// "Alternate and AV Switch Setting" の項の図を見てもらうと、Memory上から
	// TSデータを出力する経路は図中の P4 で、P4のTS出力経路が有効になるように
	// するには、下の表を見てもらうと ALT[02]=0 ALT[03]=1 AVSW[03]=0 となって
	// いるので、そのように設定を行う。refs #38 @ vanhalen
	// (というか MMAC_PLAYBACK と MMAC_DEMUX のAPIを叩いたら、この辺りのレジスタ
	//  設定はドライバで設定してくれるようになってるか、ちゃんとAPIマニュアルに
	//  書いておいて欲しい...)
	alt  &= ~(0x03LU <<  2);
#if 1
	/*
	 * Dione電源オフ中は TS出力をLOW固定(NOR ROM Address outputモード)にしておく(refs #63)
	 *   EMMA起動時はDioneは電源オフなので、TS出力しないこと。
	 */
#else
	alt  |= 0x02LU <<  2; // ALT[3:2] = 10b
#endif
	avsw &= ~(0x01LU <<  3);                       // AVSW[3] = 0b
	
	emma_writel(alt, BHIF_ALTERNATE0);
	emma_writel(avsw, BHIF_AVSWITCH);
	
	/* -----------------------------------------------------
	 * 未接続のUSB過電流検出ピンが誤検出しないようにしておく
	 * ----------------------------------------------------- */
	// ↑で過電流検出＆制御ピンGPIO38~41のPINファンクションをGPIOに設定してあります
	// そして、過電流検出ピンの極性を Low Active にしておきます
	// (ルネサスとのEMMA3SEの Q&A HW No.55 を参照)
	dat = emma_readl(FLIP_USB_CTR) & (~(0x3LU << 16));
	emma_writel(dat, FLIP_USB_CTR);
	
	/* --------------------------------------------
	 * DVB-T復調器は使わないので スリープさせておく
	 * -------------------------------------------- */
	dat = emma_readl(BHIF_DVBTC_CTRL_0) & (~(0x1LU << 4));
	emma_writel(dat, BHIF_DVBTC_CTRL_0);
	dat = emma_readl(BHIF_DVBTC_CTRL_1) & (~(0x1LU << 4));
	emma_writel(dat, BHIF_DVBTC_CTRL_1);
	dat = emma_readl(BHIF_DVBTC_CTRL_2) & (~(0x1LU << 4));
	emma_writel(dat, BHIF_DVBTC_CTRL_2);
}

/*
 * static void _setup_gpio(void) 内で行っている EtherMAC の PHY のリセットを解除する
 */
static void _free_eth_phy_reset(void)
{
#if (RST_ETHPHY_GPIO_NUM < 32)
	unsigned long dat = emma_readl(ZPIO_DATA0);
	dat |= 0x01LU << RST_ETHPHY_GPIO_NUM;
	emma_writel(dat, ZPIO_DATA0);
#else
	unsigned long dat = emma_readl(ZPIO_DATA1);
	dat |= 0x01LU << (RST_ETHPHY_GPIO_NUM - 32);
	emma_writel(dat, ZPIO_DATA1);
#endif
}

#define PIX_LED_BLACK  (0)
#define PIX_LED_RED    (1)
#define PIX_LED_GREEN  (2)
#define PIX_LED_ORANGE (3)
static void _set_led(int status, int tv)
{
	unsigned long dat;
	
	// GPIO[19~22]
	dat = emma_readl(ZPIO_DATA0);
	dat &= ~(0x0fLU << 19);
	
	switch(status){	// GPIO[19~20]
	case PIX_LED_BLACK:
		// GPIO[19~20] : LED1_A=H(Off), LED1_B=H(Off)
		dat |= 0x03LU << 19;
		break;
	case PIX_LED_RED:
		// GPIO[19~20] : LED1_A=H(Off), LED1_B=L(On)
		dat |= 0x01LU << 19;
	default:
		break;
	case PIX_LED_GREEN:
		// GPIO[19~20] : LED1_A=L(On), LED1_B=H(Off)
		dat |= 0x02LU << 19;
		break;
	case PIX_LED_ORANGE:
		// GPIO[19~20] : LED1_A=L(On), LED1_B=L(On)
		break;
	}
	switch(tv){	// GPIO[21~22]
	case PIX_LED_BLACK:
		// GPIO[21~22] : LED2_A=H(Off), LED2_B=H(Off)
		dat |= 0x03LU << 21;
		break;
	case PIX_LED_RED:
		// GPIO[21~22] : LED2_A=H(Off), LED2_B=L(On)
		dat |= 0x01LU << 21;
	default:
		break;
	case PIX_LED_GREEN:
		// GPIO[21~22] : LED2_A=L(On), LED2_B=H(Off)
		dat |= 0x02LU << 21;
		break;
	case PIX_LED_ORANGE:
		// GPIO[21~22] : LED2_A=L(On), LED2_B=L(On)
		break;
	}
	emma_writel(dat, ZPIO_DATA0);
}
static void _set_led_pattern(int lightPattern)
{
	unsigned long dat;
	
	switch(lightPattern){
	case PIX_LED_PATTERN_PON_NORMAL:
	case PIX_LED_PATTERN_PON_ACTIVE:
	case PIX_LED_PATTERN_PON_ACTIVE_NO_SESSION:
	default:
		// ステータスLED = 赤点灯, TV_LED  = 消灯
		_set_led(PIX_LED_RED, PIX_LED_BLACK);
		break;
	case PIX_LED_PATTERN_PON_FWUP:
		// ステータスLED = 橙点灯,  TV_LED = 橙点灯
		_set_led(PIX_LED_ORANGE, PIX_LED_ORANGE);
		break;
	}
}

static void _power_on_dione(void)
{
	return;
}

