/*
 * GPIOの初期設定 for Pixela製XT046(コアボード)基板 by H.HASEGAWA @ 2012.03.01 refs #18 #25
 * (\\Odessa2\hav\66_AirTuner2(DMS,Rec)\04_ハードウェア\00_回路\ポートマップ
 *  XT046制御仕様rev01_20120228.pdf を参照)
 */

#define ETHER_RST_L_GPIO_NUM	52

static void _setup_gpio(void)
{
	unsigned long avsw, alt, dat, dir;
	
	/* ------------
	 * GPIO初期設定
	 * ------------ */
	dat = dir = 0;
	// GPIO[ 0~ 4] : (O) (未使用)=L x 5
	dir |= 0x1fLU;
	// GPIO[    5] : (O) DEMOD_RST_L=L
	dir |= 0x01LU <<  5;
	// GPIO[ 6~29] : (O) (未使用)=L x 24
	dir |= 0x3fffffc0;
	// GPIO[30~31] : (I) (シェアピンで 他機能(BCASカードI/F)として使用)
	;
	
	emma_writel((~dir), ZPIO_MASK0); // 入力ピンのDATA値が書き変わらないようにMaskする！
	emma_writel(dat, ZPIO_DATA0);
	emma_writel(dir, ZPIO_DIR0);
	
	dat = dir = 0;
	// GPIO[32~35] : (O) (未使用)=L x 4
	dir |= 0x0fLU << (32 - 32);
	// GPIO[36~37] : (I) (シェアピンで 他機能(FUART0)として使用)
	;
	// GPIO[38~39] : (I) (シェアピンで 他機能(USB1のOC検出/制御)として使用)
	;
	// GPIO[40~41] : (O) (未使用)=L, AUDIO_SHUT_L=L
	dir |= 0x03LU << (40 - 32);
	// GPIO[42~46] : (I) (シェアピンで 他機能(TS入力)として使用)
	dir &= ~(0x1fLU << (42 - 32));
	// GPIO[47~48] : (O) (未使用)=L x 2
	dir |= 0x03LU << (47 - 32);
	// GPIO[49~51] : (I) (シェアピンで 他機能(BCASカードI/F)として使用)
	dir &= ~(0x07LU << (49 - 32));
	// GPIO[   52] : (O) ETHER_RST_L=L
	dir |=   0x01LU << (ETHER_RST_L_GPIO_NUM - 32);
	// GPIO[53~59] : (O) (未使用)=L x 7
	dir |= 0x7fLU << (53 - 32);
	
	emma_writel((~dir), ZPIO_MASK1); // 入力ピンのDATA値が書き変わらないようにMaskする！
	emma_writel(dat, ZPIO_DATA1);
	emma_writel(dir, ZPIO_DIR1);
	
	/* ---------------------
	 * PINファンクション設定
	 * --------------------- */
	/*
	 * データシート(R19DS0036ED0104_EMMA3SE_20111208.pdf)の 2.5.2 Shared Interface Conflicts 項の
	 * Table 19 PPORT (Programmable Pin Port) の表を見ながら、GPIOのPINファンクション設定を行う
	 * Table 19 中の "Option xx" の "xx" の番号は ALTレジスタ の bit位置 に対応しています
	 */
	alt = emma_readl(BHIF_ALTERNATE0);
	avsw = emma_readl(BHIF_AVSWITCH) & 0x0000000fLU;
	
	// GPIO[ 0~ 1] を GPIO へ          by ALT[19:18] = 00b
	// GPIO[ 2~ 5] を GPIO へ          by ALT[22,18] = 0b,0b
	// GPIO[ 6~ 8] を GPIO へ          by ALT[18] = 0b
	// GPIO[ 9~11] を GPIO へ          by ALT[19:18] = 00b
	alt  &= ~(0x01LU << 22);                       // ALT[22] = 0b
	alt  &= ~(0x03LU << 18);                       // ALT[19:18] = 00b
	
	// GPIO[   12] は Not shared
	// GPIO[13~16] を GPIO へ          by ALT[21:20] = 00b
	// GPIO[   17] は Not shared
	// GPIO[18~19] を GPIO へ          by ALT[21] = 0b
	// GPIO[20~23] を GPIO へ          by ALT[21:20] = 00b
	// GPIO[   24] を GPIO へ          by ALT[21:20,0] = 00b,0b
	// GPIO[25~29] を GPIO へ          by ALT[21:20,4,1:0] = 00b,0b,00b
	alt  &= ~(0x03LU << 20);                       // ALT[21:20] = 00b
	alt  &= ~(0x01LU <<  4);                       // ALT[4] = 0b
	alt  &= ~(0x03LU);                             // ALT[1:0] = 00b
	
	// GPIO[30~31] を BCASカードI/F へ by ALT[5] = 1b
	// GPIO[47~48] を GPIO へ          by ALT[15,12] = 0b,0b
	// GPIO[49~51] を BCASカードI/F へ by ALT[15,5] = 0b,1b
	// GPIO[   52] を GPIO へ          by ALT[15] = 0b
	alt  &= ~(0x01LU << 15);                       // ALT[15] = 0b
	alt  &= ~(0x01LU << 12);                       // ALT[12] = 0b
	                         alt  |= 0x01LU <<  5; // ALT[5] = 1b
	
	// GPIO[   32] は Not shared
	// GPIO[   33] を GPIO へ          by ALT[11] = 0b
	alt  &= ~(0x01LU << 11);                       // ALT[11] = 0b
	
	// GPIO[34~35] を GPIO へ          by ALT[24,10] = 0b,0b
	alt  &= ~(0x01LU << 24);                       // ALT[24] = 0b
	alt  &= ~(0x01LU << 10);                       // ALT[10] = 0b
	
	// GPIO[36~37] を FUART0 へ        by ALT[9] = 1b
	                         alt  |= 0x01LU <<  9; // ALT[9] = 1b
	
	// GPIO[38~39] を USB1のOC検出/制御 へ by ALT[7] = 1b
	                         alt  |= 0x01LU <<  7; // ALT[7] = 1b
	
	// GPIO[40~41] を GPIO へ          by ALT[8] = 0b
	alt  &= ~(0x01LU <<  8);                       // ALT[8] = 0b
	
	// GPIO[42~46] を TS入力 へ        by ALT[17] = 1b
	                         alt  |= 0x01LU << 17; // ALT[17] = 1b
	// EMMA内部の Stream Proc. の Serial in 0 へ TS1入力ピン(STS1)に入力されるTSデータを入力
	                         avsw |= 0x01LU;       // AVSW[0] = 1b
	
	// GPIO[47~52] の設定は↑の GPIO[30~31] の設定を参照
	
	// GPIO[53~57] を GPIO へ          by ALT[26:25] = 00b
	// GPIO[58~59] は Not shared
	alt  &= ~(0x03LU << 25);                       // ALT[26:25] = 00b
	
	emma_writel(alt, BHIF_ALTERNATE0);
	emma_writel(avsw, BHIF_AVSWITCH);
	
	/* -----------------------------------------------------
	 * 未接続のUSB過電流検出ピンが誤検出しないようにしておく
	 * ----------------------------------------------------- */
	// ↑でUSB2の過電流検出＆制御ピンGPIO40~41のPINファンクションをGPIOに設定してあります
	// そして、USB2の過電流検出ピンの極性を Low Active にしておきます
	// (ルネサスとのEMMA3SEの Q&A HW No.55 を参照)
	dat = emma_readl(FLIP_USB_CTR) & (~(0x1LU << 17));
	emma_writel(dat, FLIP_USB_CTR);
	
	/* --------------------------------------------
	 * DVB-T復調器は使わないので スリープさせておく
	 * -------------------------------------------- */
	dat = emma_readl(BHIF_DVBTC_CTRL_0) & (~(0x1LU << 4));
	emma_writel(dat, BHIF_DVBTC_CTRL_0);
	dat = emma_readl(BHIF_DVBTC_CTRL_1) & (~(0x1LU << 4));
	emma_writel(dat, BHIF_DVBTC_CTRL_1);
	dat = emma_readl(BHIF_DVBTC_CTRL_2) & (~(0x1LU << 4));
	emma_writel(dat, BHIF_DVBTC_CTRL_2);
}

/*
 * static void _setup_gpio(void) 内で行っている EtherMAC の PHY のリセットを解除する
 */
static void _free_eth_phy_reset(void)
{
#if (ETHER_RST_L_GPIO_NUM < 32)
	unsigned long dat = emma_readl(ZPIO_DATA0);
	dat |= 0x01LU << ETHER_RST_L_GPIO_NUM;
	emma_writel(dat, ZPIO_DATA0);
#else
	unsigned long dat = emma_readl(ZPIO_DATA1);
	dat |= 0x01LU << (ETHER_RST_L_GPIO_NUM - 32);
	emma_writel(dat, ZPIO_DATA1);
#endif
}

static void _set_led_pattern(int lightPattern)
{
	// 何もしない
}

static void _power_on_dione(void)
{
	return;
}

