/*
 * (C) Copyright 2003-2010
 * Thomas.Lange@corelatus.se
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <netdev.h>
#include <asm/arch-emma/emma.h>
#include <configs/emma3slx.h>
#include <asm/mipsregs.h>
#include <asm/reboot.h>

/* FLASH chips info */
#if defined(CONFIG_SYS_FLASH_PROTECTION)
extern flash_info_t flash_info[CONFIG_SYS_MAX_FLASH_BANKS];
#endif
extern int candy_enet_initialize(bd_t *bis);

void _machine_restart(void)
{
	void (*f)(void) = (void *)TEXT_BASE;

	f();
}

phys_size_t initdram(int board_type)
{
	return CONFIG_SYS_SDRAM_SIZE;
}

int checkboard(void)
{
	u32 bhif_version;
	u32 bhif_gen;

	/* read BHIF version register */
	bhif_version = emma_readl(BHIF_VERSION);
	bhif_gen = emma_readl(BHIF_GENERATION);

	printf("Board: EMMA3S series ");
	printf("(CPU Speed %d MHz, ", (int)CPU_CLOCK_RATE / 1000000);
	printf("DRAM Size %d MB)\n", (int)(CONFIG_SYS_SDRAM_SIZE >> 20));
	printf("BHIF version = 0x%08x (", (bhif_version & 0x0000ffff));
	if (bhif_gen == 0x02000012)
		printf("EMMA3SL/P ");
	else if (bhif_gen == 0x02000013)
		printf("EMMA3SE/P ");
	else
		printf("unknown device ");

	/* BHIF Major version, Minor version */
	printf("%d.%d)\n", (bhif_version & 0x000000f0)>>4,
			(bhif_version & 0x0000000f));

	return 0;
}

/* Monitor, Linux kernel protection ON by default */
#if defined(CONFIG_SYS_FLASH_PROTECTION)
static void setup_flashrom_protection(void)
{

	/* Monitor protection ON by default */
	flash_protect(FLAG_PROTECT_SET,
			CONFIG_SYS_MONITOR_BASE,
			CONFIG_SYS_MONITOR_BASE + CONFIG_SYS_MONITOR_LEN - 1,
			&flash_info[0]);

	/* Linux kernel protection ON by default */
	flash_protect(FLAG_PROTECT_SET,
			CONFIG_SYS_KERNEL_ADDR,
			CONFIG_SYS_KERNEL_ADDR + CONFIG_SYS_KERNEL_SIZE - 1,
			&flash_info[0]);

}
#endif

#ifdef CONFIG_SPI_FLASH
static void setup_spi_flash(void)
{
	u32 reg;

#if 0	/* moved to lowlevel_init() */
	reg = emma_readl(BROI_ROM_MODE);
	#if 0
	reg |= 0x00030000;		/* CS0 */
	#else
	reg |= 0x000c0000;		/* CS1 */
	#endif
	emma_writel(reg, BROI_ROM_MODE);
#endif

	reg = emma_readl(CHIF_PERICLK_SEL) & ~0x03000000;
#if 0 /* use default clock */
	reg |= 0x01000000;		/* SFCLK=72MHz */
#else
	reg |= 0x00000000;		/* SFCLK=32MHz */
#endif
	emma_writel(reg, CHIF_PERICLK_SEL);
}
#endif

/* miscellaneous platform dependent initialisations */
int misc_init_r(void)
{
	u32 bootmode = emma_readl(BHIF_STRAP_0) & 0x06;

	if (bootmode == 0x00) {	/* NOR-ROM boot */
#if defined(CONFIG_SYS_FLASH_PROTECTION)
		/* Monitor, Linux kernel protection ON by default */
		setup_flashrom_protection();
#endif
#ifdef CONFIG_SPI_FLASH
		setup_spi_flash();
#endif
	}

	return 0;
}

#ifdef CONFIG_CMD_NET
int board_eth_init(bd_t *bis)
{
#if defined(CONFIG_NEC_CANDY)
	candy_enet_initialize(bis);
#endif
	return 0;
}
#endif

#ifdef CONFIG_SYS_I2C_INIT_BOARD
void i2c_init_board(void)
{
}
#endif

