/*
 * Memory sub-system initialization code for EMMA3SLx development board.
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <config.h>
#include <version.h>
#include <asm/regdef.h>
#include <asm/mipsregs.h>

/**********************************************************************/
#define MEM_INIT_WAIT_SCLK	(2000 * 3)

/*
 *
 */
#define MIFA_DLFSL		0x0cf4
#define MIFA_IOLHFIX		0x0cb0

/*
 * ROM I/F Init DATA
 */
#if defined(CONFIG_NAND_BOOT)
#define ROM_MODE_DATA		0x000b0811	/* 8MB, CS0=Serial, CS1=NAND */
#define ROM_IF_DATA		0x01000403
#elif defined(CONFIG_SERIAL_BOOT)
#define ROM_MODE_DATA		0x00030000	/* 4MB, CS0=Serial CS1=none */
#define ROM_IF_DATA		0x00000001
#else
#define ROM_MODE_DATA		0x00080811	/* 8MB, CS0=NOR, CS1=NAND */
#define ROM_IF_DATA		0x00000003
#endif
#define ROM_ACCESS_TIME_DATA	0x00000000

	.text
	.set noreorder
	.set mips32

	.globl	lowlevel_init
lowlevel_init:

	.macro	REGWR base, offset, val
	li	t1, \val
	sw	t1, \offset(\base)
	.endm

	/* PBRD Init. */
	li	t0, 0xb0100000
	REGWR	t0 0x0010 0xc0

	/*
	 * ROM I/F Init.
	 */
	li	t0, 0xb0000000
	REGWR	t0 0x0400 ROM_MODE_DATA
/*	REGWR	t0 0x0404 ROM_ACCESS_TIME_DATA */
	REGWR	t0 0x0408 ROM_IF_DATA

ROM_INIT_END:

	li	t0, 0xb0000000
	lw	t1, 0x0028(t0)
	/* Alternate0 set USB, UART0 */
	li	t2, 0x780
	or	t1, t1, t2
#ifdef CONFIG_NAND_BOOT
	/* Alternate0 set NAND */
	li	t2, 0x8000
	or	t1, t1, t2
#endif
	sw	t1, 0x0028(t0)

PORT_INIT_END:

	/*
	  checks whether the fetch address is RAM(0x8XXXXXXX,0xAXXXXXXX)
	  or ROM(0x9XXXXXXX,0xBXXXXXXX).
	  If the address is RAM, memory initialization should skip.
	*/
	li	t0, 0x10000000
	and	t1, t0, ra
	xor	t1, t1, t0
	bnez	t1, MEM_INIT_END
	nop

MEM_INIT_START:
	/*
	 * Configure EMMA DDR memory-bus
	 */

	li	t0, 0xb0000000
	/*
	 * (1-1) DDR2 macro Initialize
	*/
	REGWR	t0 0x0cf8 0xe1e1e1e1
	REGWR	t0 0x0cfc 0x25252525
	REGWR	t0 0x0cf4 0x00000000
	REGWR	t0 0x0cec 0x0fff0000
	REGWR	t0 0x0cf0 0x00000000
	REGWR	t0 0x0ce8 0x00000001

	/* 0x1 = lock state */
	li	t2, 0x1
MEM_LOCK_WAIT_A:
	lw	t1, MIFA_DLFSL(t0)
	and	t1, t2
	bne	t1, t2, MEM_LOCK_WAIT_A
	nop

	/* 0x1 = FIX */
	li	t2, 0x1
MEM_IOLHFIX_WAIT_A:
	lw	t1, MIFA_IOLHFIX(t0)
	and	t1, t2
	bne	t1, t2, MEM_IOLHFIX_WAIT_A
	nop

	REGWR	t0 0x0ce0 0x00000001
	REGWR	t0 0x0ce4 0x00f00301

	/* wait for 200us */
	li	t2, 0x2000
WAIT_200_A:
	bne	t2, zero, WAIT_200_A
	addi	t2, -1

	/*
	 * Set MIF CONFIG
	*/
	/* (1-2-6) CS0 1Gb(x16) CL5 32bit bus */
	REGWR	t0 0x0c04 0xb4eb1d66
	REGWR	t0 0x0c10 0x15483249
	REGWR	t0 0x0c14 0x40d50126

	/*
	 * DDR2 SDRAM initialization
	*/
	REGWR	t0 0x0c08 0x00000020

	/* wait for 400ns */
	.rept 16
		nop
	.endr

	REGWR	t0 0x0c08 0x20000004
	REGWR	t0 0x0c08 0x00020002
	REGWR	t0 0x0c08 0x00030002
	REGWR	t0 0x0c08 0x00010002
	REGWR	t0 0x0c08 0x5a900002

	/* wait for 200 SCLK */
	li	t2, MEM_INIT_WAIT_SCLK
MEM_INIT_WAIT_A:
	bne	t2, zero, MEM_INIT_WAIT_A
	addi	t2, -1

	REGWR	t0 0x0c08 0x20000004
	REGWR	t0 0x0c08 0x00000001
	REGWR	t0 0x0c08 0x00000001

	/* wait for 100ns */
	.rept 4
		nop
	.endr

	REGWR	t0 0x0c08 0x52900002
	REGWR	t0 0x0c08 0x1c010002
	REGWR	t0 0x0c08 0x00b10002

	/* Set Refresh Counter */
	REGWR	t0 0x0c0c 0x00000162

MEM_INIT_END:
	jr	ra
	nop

