/*
 * (C) Copyright 2008
 * Stuart Wood, Lab X Technologies <stuart.wood@labxtechnologies.com>
 *
 * (C) Copyright 2004
 * Jian Zhang, Texas Instruments, jzhang@ti.com.

 * (C) Copyright 2000-2006
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2001 Sysgo Real-Time Solutions, GmbH <www.elinos.com>
 * Andreas Heppel <aheppel@sysgo.de>

 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/* #define DEBUG */

#include <common.h>
#include <command.h>
#include <environment.h>
#include <linux/stddef.h>
#include <malloc.h>
#include <nand.h>
#include <asm/errno.h>

#if defined(CONFIG_CMD_SAVEENV) && defined(CONFIG_CMD_NAND)
#define CMD_SAVEENV
#elif defined(CONFIG_ENV_OFFSET_REDUND)
#error Cannot use CONFIG_ENV_OFFSET_REDUND without CONFIG_CMD_SAVEENV & CONFIG_CMD_NAND
#endif

	#if 1 /* Add by hiroshi.hasegawa@pixela.jp 2012.02.20 */
#if defined(CONFIG_CMD_DELENV) && defined(CONFIG_CMD_NAND)
#define CMD_DELENV
#endif
	#endif

#if defined(CONFIG_ENV_SIZE_REDUND) && (CONFIG_ENV_SIZE_REDUND != CONFIG_ENV_SIZE)
#error CONFIG_ENV_SIZE_REDUND should be the same as CONFIG_ENV_SIZE
#endif

#ifndef CONFIG_ENV_RANGE
#define CONFIG_ENV_RANGE	CONFIG_ENV_SIZE
#endif

/* references to names in env_common.c */
extern uchar default_environment[];

char * env_name_spec = "NAND";


#if defined(ENV_IS_EMBEDDED)
extern uchar environment[];
env_t *env_ptr = (env_t *)(&environment[0]);
#elif defined(CONFIG_NAND_ENV_DST)
env_t *env_ptr = (env_t *)CONFIG_NAND_ENV_DST;
#else /* ! ENV_IS_EMBEDDED */
env_t *env_ptr = 0;
#endif /* ENV_IS_EMBEDDED */
#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
static u_char *_rw_page_buf = NULL; /* NAND Flashの1page分のデータの読み書き用バッファ */
#endif


/* local functions */
#if !defined(ENV_IS_EMBEDDED)
static void use_default(void);
#endif
#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
static void _del_env_ethaddr(char *ethaddr, int ethaddr_len);
static void _save_and_set_env_ethaddr(char *ethaddr, int ethaddr_len);
static void _load_and_set_env_ethaddr(env_t *env_ptr);
static int _check_all_ff(u_char *data, int data_len);
static char *_load_ethaddr(void);
#endif

DECLARE_GLOBAL_DATA_PTR;

uchar env_get_char_spec (int index)
{
	return ( *((uchar *)(gd->env_addr + index)) );
}


/* this is called before nand_init()
 * so we can't read Nand to validate env data.
 * Mark it OK for now. env_relocate() in env_common.c
 * will call our relocate function which does the real
 * validation.
 *
 * When using a NAND boot image (like sequoia_nand), the environment
 * can be embedded or attached to the U-Boot image in NAND flash. This way
 * the SPL loads not only the U-Boot image from NAND but also the
 * environment.
 */
int env_init(void)
{
#if defined(ENV_IS_EMBEDDED) || defined(CONFIG_NAND_ENV_DST)
	int crc1_ok = 0, crc2_ok = 0;
	env_t *tmp_env1;

#ifdef CONFIG_ENV_OFFSET_REDUND
	env_t *tmp_env2;

	tmp_env2 = (env_t *)((ulong)env_ptr + CONFIG_ENV_SIZE);
	crc2_ok = (crc32(0, tmp_env2->data, ENV_SIZE) == tmp_env2->crc);
#endif

	tmp_env1 = env_ptr;

	crc1_ok = (crc32(0, tmp_env1->data, ENV_SIZE) == tmp_env1->crc);

	if (!crc1_ok && !crc2_ok) {
		gd->env_addr  = 0;
		gd->env_valid = 0;

		return 0;
	} else if (crc1_ok && !crc2_ok) {
		gd->env_valid = 1;
	}
#ifdef CONFIG_ENV_OFFSET_REDUND
	else if (!crc1_ok && crc2_ok) {
		gd->env_valid = 2;
	} else {
		/* both ok - check serial */
		if(tmp_env1->flags == 255 && tmp_env2->flags == 0)
			gd->env_valid = 2;
		else if(tmp_env2->flags == 255 && tmp_env1->flags == 0)
			gd->env_valid = 1;
		else if(tmp_env1->flags > tmp_env2->flags)
			gd->env_valid = 1;
		else if(tmp_env2->flags > tmp_env1->flags)
			gd->env_valid = 2;
		else /* flags are equal - almost impossible */
			gd->env_valid = 1;
	}

	if (gd->env_valid == 2)
		env_ptr = tmp_env2;
	else
#endif
	if (gd->env_valid == 1)
		env_ptr = tmp_env1;

	gd->env_addr = (ulong)env_ptr->data;

#else /* ENV_IS_EMBEDDED || CONFIG_NAND_ENV_DST */
	gd->env_addr  = (ulong)&default_environment[0];
	gd->env_valid = 1;
#endif /* ENV_IS_EMBEDDED || CONFIG_NAND_ENV_DST */

	return (0);
}

#ifdef CMD_SAVEENV
/*
 * The legacy NAND code saved the environment in the first NAND device i.e.,
 * nand_dev_desc + 0. This is also the behaviour using the new NAND code.
 */
int writeenv(size_t offset, u_char *buf)
{
	size_t end = offset + CONFIG_ENV_RANGE;
	size_t amount_saved = 0;
	size_t blocksize, len;

	u_char *char_ptr;

	blocksize = nand_info[0].erasesize;
	len = min(blocksize, CONFIG_ENV_SIZE);

	while (amount_saved < CONFIG_ENV_SIZE && offset < end) {
		if (nand_block_isbad(&nand_info[0], offset)) {
			offset += blocksize;
		} else {
			char_ptr = &buf[amount_saved];
			if (nand_write(&nand_info[0], offset, &len,
					char_ptr))
				return 1;
			offset += blocksize;
			amount_saved += len;
		}
	}
	if (amount_saved != CONFIG_ENV_SIZE)
		return 1;

	return 0;
}
#ifdef CONFIG_ENV_OFFSET_REDUND
int saveenv(void)
{
	int ret = 0;
	nand_erase_options_t nand_erase_options;
	char ethaddr[] = PIX_ENV_ETHADDR_DEFAULT; /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */

	env_ptr->flags++;

	nand_erase_options.length = CONFIG_ENV_RANGE;
	nand_erase_options.quiet = 0;
	nand_erase_options.jffs2 = 0;
	nand_erase_options.scrub = 0;

	if (CONFIG_ENV_RANGE < CONFIG_ENV_SIZE)
		return 1;
	if(gd->env_valid == 1) {
		puts ("Erasing redundant Nand...\n");
		nand_erase_options.offset = CONFIG_ENV_OFFSET_REDUND;
		if (nand_erase_opts(&nand_info[0], &nand_erase_options))
			return 1;

		puts ("Writing to redundant Nand... ");
		_del_env_ethaddr(ethaddr, sizeof(ethaddr)); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
		ret = writeenv(CONFIG_ENV_OFFSET_REDUND, (u_char *) env_ptr);
	} else {
		puts ("Erasing Nand...\n");
		nand_erase_options.offset = CONFIG_ENV_OFFSET;
		if (nand_erase_opts(&nand_info[0], &nand_erase_options))
			return 1;

		puts ("Writing to Nand... ");
		_del_env_ethaddr(ethaddr, sizeof(ethaddr)); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
		ret = writeenv(CONFIG_ENV_OFFSET, (u_char *) env_ptr);
	}
	if (ret) {
		if (ethaddr[0]) { /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
			setenv("ethaddr", ethaddr);
		}
		puts("FAILED!\n");
		return 1;
	}

	puts ("done\n");
	_save_and_set_env_ethaddr(ethaddr, sizeof(ethaddr)); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
	gd->env_valid = (gd->env_valid == 2 ? 1 : 2);
	return ret;
}
#else /* ! CONFIG_ENV_OFFSET_REDUND */
int saveenv(void)
{
	int ret = 0;
	nand_erase_options_t nand_erase_options;
	char ethaddr[] = PIX_ENV_ETHADDR_DEFAULT; /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */

	nand_erase_options.length = CONFIG_ENV_RANGE;
	nand_erase_options.quiet = 0;
	nand_erase_options.jffs2 = 0;
	nand_erase_options.scrub = 0;
	nand_erase_options.offset = CONFIG_ENV_OFFSET;

	if (CONFIG_ENV_RANGE < CONFIG_ENV_SIZE)
		return 1;
	puts ("Erasing Nand...\n");
	if (nand_erase_opts(&nand_info[0], &nand_erase_options))
		return 1;

	puts ("Writing to Nand... ");
	/* NAND Flashに環境変数を保存する前に、MACアドレス値は別途に保存するので
	 * いったん環境変数からMACアドレス値を取り除く */
	_del_env_ethaddr(ethaddr, sizeof(ethaddr)); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
	if (writeenv(CONFIG_ENV_OFFSET, (u_char *) env_ptr)) {
		/* 環境変数からいったん取り除いていたMACアドレス値を戻す */
		if (ethaddr[0]) { /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
			setenv("ethaddr", ethaddr);
		}
		puts("FAILED!\n");
		return 1;
	}

	puts ("done\n");
	/* MACアドレス値を環境変数の保存領域とは別の領域(mtd1の最終Block)に保存して
	 * 環境変数からいったん取り除いていたMACアドレス値を戻す */
	_save_and_set_env_ethaddr(ethaddr, sizeof(ethaddr)); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
	return ret;
}
#endif /* CONFIG_ENV_OFFSET_REDUND */
#endif /* CMD_SAVEENV */

	#if 1 /* Add by hiroshi.hasegawa@pixela.jp 2012.02.20 */
#ifdef CMD_DELENV
#ifdef CONFIG_ENV_OFFSET_REDUND
int delenv(void)
{
	int ret = 0;
	nand_erase_options_t nand_erase_options;

	env_ptr->flags++;

	nand_erase_options.length = CONFIG_ENV_RANGE;
	nand_erase_options.quiet = 0;
	nand_erase_options.jffs2 = 0;
	nand_erase_options.scrub = 0;

	if (CONFIG_ENV_RANGE < CONFIG_ENV_SIZE)
		return 1;

	puts ("Erasing redundant Nand...\n");
	nand_erase_options.offset = CONFIG_ENV_OFFSET_REDUND;
	if (nand_erase_opts(&nand_info[0], &nand_erase_options))
		return 1;

	puts ("Erasing Nand...\n");
	nand_erase_options.offset = CONFIG_ENV_OFFSET;
	if (nand_erase_opts(&nand_info[0], &nand_erase_options))
		return 1;

	puts ("done\n");

	puts ("using default environment\n");
	set_default_env();

	return ret;
}
#else /* ! CONFIG_ENV_OFFSET_REDUND */
int delenv(void)
{
	int ret = 0;
	nand_erase_options_t nand_erase_options;

	nand_erase_options.length = CONFIG_ENV_RANGE;
	nand_erase_options.quiet = 0;
	nand_erase_options.jffs2 = 0;
	nand_erase_options.scrub = 0;
	nand_erase_options.offset = CONFIG_ENV_OFFSET;

	if (CONFIG_ENV_RANGE < CONFIG_ENV_SIZE)
		return 1;
	puts ("Erasing Nand...\n");
	if (nand_erase_opts(&nand_info[0], &nand_erase_options))
		return 1;

	puts ("done\n");

	puts ("using default environment\n");
	set_default_env();

	return ret;
}
#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.03.26 refs #41 @ vanhalen */
/* FWアップデート起動用の環境変数を消去 */
int delenv_u(void)
{
	int ret = 0;
	nand_erase_options_t nand_erase_options;

	nand_erase_options.length = CONFIG_ENV_RANGE;
	nand_erase_options.quiet = 0;
	nand_erase_options.jffs2 = 0;
	nand_erase_options.scrub = 0;
	nand_erase_options.offset = (128LU*1024*1024) <= nand_info[0].size ?
	                            CONFIG_ENV_UPDATE_OFFSET_1GB : CONFIG_ENV_UPDATE_OFFSET_512MB;

	if (CONFIG_ENV_RANGE < CONFIG_ENV_SIZE)
		return 1;
	puts ("Erasing Nand...\n");
	if (nand_erase_opts(&nand_info[0], &nand_erase_options))
		return 1;

	puts ("done\n");

	return ret;
}
#endif /* Add by hiroshi.hasegawa@pixela.jp 2012.03.26 refs #41 @ vanhalen */
#endif /* CONFIG_ENV_OFFSET_REDUND */
#endif /* CMD_DELENV */
	#endif /* Add by hiroshi.hasegawa@pixela.jp 2012.02.20 */

int readenv (size_t offset, u_char * buf)
{
	size_t end = offset + CONFIG_ENV_RANGE;
	size_t amount_loaded = 0;
	size_t blocksize, len;
	static int _overwrite_default_env = 0;

	u_char *char_ptr;

	blocksize = nand_info[0].erasesize;
	if (!blocksize)
		return 1;

#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.03.09 */
	/* NAND Flashの容量によってNAND Flash上のパーティション構成が異なるため
	 * Linuxカーネルイメージの格納アドレスも異なるので、ココでNAND Flashの
	 * 容量によって、デフォルトの環境変数にセットされているLinuxカーネル
	 * イメージの格納アドレスを書き換える */
	if (!_overwrite_default_env) {
		char *ethaddr;
		if ((128LU*1024*1024) <= nand_info[0].size) {
			overwrite_default_env(PIX_ENV_NBOOT_CMD_DEFAULT, PIX_ENV_NBOOT_CMD_FOR_NAND_1GB);
		} else {
			overwrite_default_env(PIX_ENV_NBOOT_CMD_DEFAULT, PIX_ENV_NBOOT_CMD_FOR_NAND_512MB);
		}
	#if 1 /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
		/* デフォルトの環境変数にセットされているMACアドレス値を
		 * NAND Flashに書き込んであるMACアドレス値に書き換える */
		if ((ethaddr = _load_ethaddr())) {
			puts("using ethaddr wrriten at last block on mtd1\n");
			overwrite_default_env(PIX_ENV_ETHADDR_DEFAULT, ethaddr);
		} else {
			puts ("using default ethaddr\n");
		}
	#endif
		_overwrite_default_env = 1;
	}
#endif

#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.04.04 */
	/* 起動時間短縮のために、NAND Flash上の環境変数保存領域に保存されている“全データ”を
	 * 読み出してからチェックするのではなく、保存領域の先頭1pageだけを読み込んで、その先頭
	 * 16byte全部が0xffならば NAND Flash上に環境変数は保存されていないものとする */
	while (offset < end) {
		if (nand_block_isbad(&nand_info[0], offset)) {
			offset += blocksize;
		} else {
			/* 1pageだけ読み込み */
			len = nand_info[0].writesize;
			char_ptr = buf;
			if (nand_read(&nand_info[0], offset, &len, char_ptr) == 0) {
				/* 先頭16byte全部が0xffなので、NAND Flash上に環境変数は保存されていない */
				if (_check_all_ff(char_ptr, 16)) {
					return 1;
				}
			}
			break;
		}
	}
#endif /* Add by hiroshi.hasegawa@pixela.jp 2012.04.04 */

	len = min(blocksize, CONFIG_ENV_SIZE);

	while (amount_loaded < CONFIG_ENV_SIZE && offset < end) {
		if (nand_block_isbad(&nand_info[0], offset)) {
			offset += blocksize;
		} else {
			char_ptr = &buf[amount_loaded];
			if (nand_read(&nand_info[0], offset, &len, char_ptr))
				return 1;
			offset += blocksize;
			amount_loaded += len;
		}
	}
	if (amount_loaded != CONFIG_ENV_SIZE)
		return 1;

	return 0;
}

#ifdef CONFIG_ENV_OFFSET_OOB
int get_nand_env_oob(nand_info_t *nand, unsigned long *result)
{
	struct mtd_oob_ops ops;
	uint32_t oob_buf[ENV_OFFSET_SIZE/sizeof(uint32_t)];
	int ret;

	ops.datbuf = NULL;
	ops.mode = MTD_OOB_AUTO;
	ops.ooboffs = 0;
	ops.ooblen = ENV_OFFSET_SIZE;
	ops.oobbuf = (void *) oob_buf;

	ret = nand->read_oob(nand, ENV_OFFSET_SIZE, &ops);
	if (ret) {
		printf("error reading OOB block 0\n");
		return ret;
	}

	if (oob_buf[0] == ENV_OOB_MARKER) {
		*result = oob_buf[1] * nand->erasesize;
	} else if (oob_buf[0] == ENV_OOB_MARKER_OLD) {
		*result = oob_buf[1];
	} else {
		printf("No dynamic environment marker in OOB block 0\n");
		return -ENOENT;
	}

	return 0;
}
#endif

#ifdef CONFIG_ENV_OFFSET_REDUND
void env_relocate_spec (void)
{
#if !defined(ENV_IS_EMBEDDED)
	int crc1_ok = 0, crc2_ok = 0;
	env_t *tmp_env1, *tmp_env2;

	tmp_env1 = (env_t *) malloc(CONFIG_ENV_SIZE);
	tmp_env2 = (env_t *) malloc(CONFIG_ENV_SIZE);

	if ((tmp_env1 == NULL) || (tmp_env2 == NULL)) {
		puts("Can't allocate buffers for environment\n");
		free (tmp_env1);
		free (tmp_env2);
		return use_default();
	}

	if (readenv(CONFIG_ENV_OFFSET, (u_char *) tmp_env1))
		puts("No Valid Environment Area Found\n");
	if (readenv(CONFIG_ENV_OFFSET_REDUND, (u_char *) tmp_env2))
		puts("No Valid Reundant Environment Area Found\n");

	crc1_ok = (crc32(0, tmp_env1->data, ENV_SIZE) == tmp_env1->crc);
	crc2_ok = (crc32(0, tmp_env2->data, ENV_SIZE) == tmp_env2->crc);

	if(!crc1_ok && !crc2_ok) {
		free(tmp_env1);
		free(tmp_env2);
		return use_default();
	} else if(crc1_ok && !crc2_ok)
		gd->env_valid = 1;
	else if(!crc1_ok && crc2_ok)
		gd->env_valid = 2;
	else {
		/* both ok - check serial */
		if(tmp_env1->flags == 255 && tmp_env2->flags == 0)
			gd->env_valid = 2;
		else if(tmp_env2->flags == 255 && tmp_env1->flags == 0)
			gd->env_valid = 1;
		else if(tmp_env1->flags > tmp_env2->flags)
			gd->env_valid = 1;
		else if(tmp_env2->flags > tmp_env1->flags)
			gd->env_valid = 2;
		else /* flags are equal - almost impossible */
			gd->env_valid = 1;

	}

	free(env_ptr);
	if(gd->env_valid == 1) {
		env_ptr = tmp_env1;
		free(tmp_env2);
	} else {
		env_ptr = tmp_env2;
		free(tmp_env1);
	}

	_load_and_set_env_ethaddr(env_ptr); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
#endif /* ! ENV_IS_EMBEDDED */
}
#else /* ! CONFIG_ENV_OFFSET_REDUND */
/*
 * The legacy NAND code saved the environment in the first NAND device i.e.,
 * nand_dev_desc + 0. This is also the behaviour using the new NAND code.
 */
void env_relocate_spec (void)
{
#if !defined(ENV_IS_EMBEDDED)
	int ret;

#if defined(CONFIG_ENV_OFFSET_OOB)
	ret = get_nand_env_oob(&nand_info[0], &nand_env_oob_offset);
	/* If unable to read environment offset from NAND OOB then fall through
	 * to the normal environment reading code below
	 */
	if (!ret)
		printf("Found Environment offset in OOB..\n");
	else
		return use_default();
#endif

#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.03.26 refs #41 @ vanhalen */
	/* FWアップデート起動用の環境変数(NAND Flashの書き換え処理を組み込んだ)が
	 * 書き込まれていたら、優先して読み込みます
	 * (FWアップデート時にu-bootでNAND Flashの書き換えを行うように trunk/tool
	 *  /make_updatefile フォルダ下にある write.sh が FWアップデート時に
	 *  /app/flashDev/tmp_update 下で実行されて、NAND Flashの書き換えを行う
	 *  処理を組み込んだ環境変数が作成され、mtd1パーティションに書き込まれ、
	 *  その環境変数をココで読み込んで起動します) */
	ret = readenv((128LU*1024*1024) <= nand_info[0].size ?
	              CONFIG_ENV_UPDATE_OFFSET_1GB : CONFIG_ENV_UPDATE_OFFSET_512MB,
	              (u_char *) env_ptr);
	if (!ret) {
		if (crc32(0, env_ptr->data, ENV_SIZE) == env_ptr->crc) {
#if defined(PRODUCT_FWIDS)
			// アップデート中かを確認 (refs #184)
			set_launchflag(LAUNCH_CAUSE_FWUP);
#endif
			puts ("using update environment on mtd1\n\n");
			return;
		}
#if defined(PRODUCT_FWIDS)
		// 起動要因を確認 (vanhalen refs #184, blur refs #127)
		if(memcmp(LAUNCH_MARKER_STR_STBY, (char *) env_ptr, LAUNCH_MARKER_LEN) == 0){
			set_launchflag(LAUNCH_CAUSE_STBY);
			puts ("exist LAUNCH marker : STANDBY\n");
		}
		if(memcmp(LAUNCH_MARKER_STR_ACTV, (char *) env_ptr, LAUNCH_MARKER_LEN) == 0){
			set_launchflag(LAUNCH_CAUSE_ACTV);
			puts ("exist LAUNCH marker : ACTIVE\n");
		}
		if(memcmp(LAUNCH_MARKER_STR_ACTV_NO_SESSION, (char *) env_ptr, LAUNCH_MARKER_LEN) == 0){
			set_launchflag(LAUNCH_CAUSE_ACTV_NO_SESSION);
			puts ("exist LAUNCH marker : ACTIVE(No Session Server)\n");
		}
		if(memcmp(LAUNCH_MARKER_STR_FWUP, (char *) env_ptr, LAUNCH_MARKER_LEN) == 0){
			// アップデート中かを確認  (refs #184)
			set_launchflag(LAUNCH_CAUSE_FWUP);
			puts ("exist LAUNCH marker : FW update\n");
		}
#endif
	}
#endif

	ret = readenv(CONFIG_ENV_OFFSET, (u_char *) env_ptr);
	if (ret)
		return use_default();

	if (crc32(0, env_ptr->data, ENV_SIZE) != env_ptr->crc)
		return use_default();

	/* NAND Flashに書き込んであるMACアドレス値を、環境変数にセット */
	_load_and_set_env_ethaddr(env_ptr); /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
#endif /* ! ENV_IS_EMBEDDED */
}
#endif /* CONFIG_ENV_OFFSET_REDUND */

#if !defined(ENV_IS_EMBEDDED)
static void use_default()
{
#if 1 /* Add by hiroshi.hasegawa@pixela.jp 2012.02.20 */
	/* 下記のようなログが出ると、異常では？と思われかねず紛らわしいので、ログを変更する */
	puts ("using default environment\n\n");
#else
	puts ("*** Warning - bad CRC or NAND, using default environment\n\n");
#endif
	set_default_env();
}
#endif

#if 1  /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */
/* 環境変数のMACアドレス値を削除 */
static void _del_env_ethaddr(char *ethaddr, int ethaddr_len)
{
	char *s = getenv("ethaddr");
	if (s) {
		memcpy(ethaddr, s, ethaddr_len);
		ethaddr[ethaddr_len - 1] = '\0'; /* 念のため終端文字をセット */
		setenv("ethaddr", NULL); /* 環境変数 ethaddr をいったん削除 */
	} else {
		ethaddr[0] = '\0';
	}
}
/* MACアドレス値をNAND Flashに書き込み＆環境変数にセット */
static void _save_and_set_env_ethaddr(char *ethaddr, int ethaddr_len)
{
#if defined(CONFIG_CMD_NAND)
	size_t offset, len;
	nand_info_t *nand = &nand_info[0];
	nand_erase_options_t nand_erase_options;
#endif /* CONFIG_CMD_NAND */

	/* MACアドレス値が引数にセットされていなければ、defaultのMACアドレス値を環境変数にセット */
	if (ethaddr[ 0] != '\0'
	 && ethaddr[ 2] == ':' && ethaddr[ 5] == ':' && ethaddr[ 8] == ':'
	 && ethaddr[11] == ':' && ethaddr[14] == ':' && ethaddr[17] == '\0') {
		setenv("ethaddr", ethaddr); /* MACアドレス値がセットされているならば、環境変数にセット */
	} else {
		puts ("using default ethaddr\n");
		setenv("ethaddr", PIX_ENV_ETHADDR_DEFAULT);
		return;
	}

#if defined(CONFIG_CMD_NAND)
	if (_rw_page_buf == NULL) {
		_rw_page_buf = malloc(nand->writesize);
	}

	/* MACアドレス値が書き込まれているmtd1パーティション([ 1]u-boot environment for update and MAC address)の
	 * 最終Blockの先頭アドレスをセット
	 * (trunk/kernel/linux-2.6.27.39/drivers/mtd/nand/nand_emma_ptt_*.h を参照) */
	if ((128LU*1024*1024) <= nand->size) {
		offset = PIX_ENV_ETHADDR_OFFSET_FOR_NAND_1GB(nand->erasesize);
	} else {
		offset = PIX_ENV_ETHADDR_OFFSET_FOR_NAND_512MB(nand->erasesize);
	}

	/* MACアドレス値を書き込み */
	while (offset) {
		/* 不良Blockならば、1つ前のBlockへ */
		if (nand_block_isbad(nand, offset)) {
			offset -= nand->erasesize;
			continue;
		}
		
		/* Block消去 */
		puts ("Erasing Nand...\n");
		nand_erase_options.length = nand->erasesize;
		nand_erase_options.quiet = 0;
		nand_erase_options.jffs2 = 0;
		nand_erase_options.scrub = 0;
		nand_erase_options.offset = offset;
		if (nand_erase_opts(nand, &nand_erase_options)) {
			/* Block消去失敗ならば、1つ前のBlockへ */
			offset -= nand->erasesize;
			continue;
		}
		
		/* 不良Blockになったならば、1つ前のBlockへ */
		if (nand_block_isbad(nand, offset)) {
			offset -= nand->erasesize;
			continue;
		}
		
		/* 書き込み */
		if (memcmp(ethaddr, PIX_ENV_ETHADDR_DEFAULT, ethaddr_len)) { /* defaultのMACアドレス値と同じならば、書き込まない */
			puts ("Writing ethaddr to last block on mtd1... ");
			memset(_rw_page_buf, 0xff, nand->writesize);
			memcpy(_rw_page_buf, ethaddr, ethaddr_len);
			len = nand->writesize;
			if (nand_write(nand, offset, &len, _rw_page_buf)) {
				/* 書き込み失敗ならば、不良Blockマーク＆1つ前のBlockへ */
				nand->block_markbad(nand, offset);
				offset -= nand->erasesize;
				continue;
			}
		}
		puts ("done\n");
		break;
	}
#endif /* CONFIG_CMD_NAND */
}
/* NAND FlashからMACアドレス値を読み出し＆環境変数にセット
 * (本当は setenv() を利用して簡単な処理にしたかったが setenv() を使う
 *  にはタイミング等に注意が必要なようで、本関数が呼ばれるタイミングで
 *  setenv() を利用して環境変数をセットすると、そのセットした環境変数が
 *  消えてしまう等の不具合が生じたため、環境変数のセット処理を自前で
 *  下記の通り用意する) */
static void _load_and_set_env_ethaddr(env_t *env_ptr)
{
	char *ethaddr, *data, *data_end;

	/* MACアドレス値がNAND Flashに書き込まれていたら、そのMACアドレス値を環境変数にセット */
	if ((ethaddr = _load_ethaddr())) {
	/*	puts("using ethaddr wrriten at last block on mtd1\n"); */
	} else {
		/* MACアドレス値が書き込まれていなかったら、defaultのMACアドレス値を環境変数にセット */
	/*	puts ("using default ethaddr\n"); */
		ethaddr = PIX_ENV_ETHADDR_DEFAULT;
	}

	/* MACアドレス値を環境変数にセット */
	data = (char *)env_ptr->data;
	data_end = data + ENV_SIZE; /* ↓環境変数の終端には NULL文字が2つセットされている！ */
	while (data != data_end && (data[0] || data[1])) {
		/* 既に環境変数にセットされているMACアドレス値を探す */
		if (data[0] == 'e' && data[1] == 't') {
			if (memcmp(data, "ethaddr=", 8) == 0) {
				/* 既にMACアドレス値が環境変数にセットされていたら
				 * NAND Flashに書き込んであるMACアドレス値に書き換える */
				data += 8;
				while (*data) {
					*data = *ethaddr;
					data++; ethaddr++;
				}
				env_ptr->crc = crc32(0, env_ptr->data, ENV_SIZE);
				return;
			}
		}
		data++;
	}

	/* 環境変数にMACアドレス値がセットないならば、追加セットする */
	if (data != data_end && data[0] == '\0') {
		memcpy(&data[1], "ethaddr=", 8);
		memcpy(&data[1 + 8], ethaddr, 17);
		memcpy(&data[1 + 8 + 17], "\0\0", 2); /* 環境変数の終端には NULL文字を2つセットする！ */
		env_ptr->crc = crc32(0, env_ptr->data, ENV_SIZE);
	}
}
/* データがすべて0xffかチェック
 * (NAND Flashは1bit化けが発生するので、それを無視するために値が'0'のbitが
 *  1つあっても、それを無視してデータがすべて0xffかチェックを行う関数) */
static int _check_all_ff(u_char *data, int data_len)
{
	u_char *data_top, mask;
	int cnt; /* 値が'0'のbit数 */

	data_top = data;
	data += data_len;
	cnt = 0;
	do {
		data--;
		if (*data != 0xff) {
			mask = 1;
			while (mask) {
				if (!(*data & mask)) {
					cnt++;
					/* '0'のbitが2つ見つかったら、これ以上のカウントは不要なので */
					if (2 <= cnt) {
						data = data_top; /* ループを抜けさせるために data == data_top へ */
						break;
					}
				}
				mask <<= 1;
			}
		}
	} while (data != data_top);

	return ((cnt <= 1) ? 1 : 0);
}
/* NAND FlashからMACアドレス値を読み出し */
static char *_load_ethaddr(void)
{
#if defined(CONFIG_CMD_NAND)
	size_t offset, len;
	nand_info_t *nand = &nand_info[0];

	if (_rw_page_buf == NULL) {
		_rw_page_buf = malloc(nand->writesize);
	}

	/* MACアドレス値が書き込まれているmtd1パーティション([ 1]u-boot environment for update and MAC address)の
	 * 最終Blockの先頭アドレスをセット
	 * (trunk/kernel/linux-2.6.27.39/drivers/mtd/nand/nand_emma_ptt_*.h を参照) */
	if ((128LU*1024*1024) <= nand->size) {
		offset = PIX_ENV_ETHADDR_OFFSET_FOR_NAND_1GB(nand->erasesize);
	} else {
		offset = PIX_ENV_ETHADDR_OFFSET_FOR_NAND_512MB(nand->erasesize);
	}

	/* 不良Blockならば、1つ前のBlockへ */
	while (offset) {
		if (nand_block_isbad(nand, offset)) {
			offset -= nand->erasesize;
		} else {
			break;
		}
	}

	/* MACアドレス値を読み出し */
	if (offset) {
		len = nand->writesize;
		if (nand_read(nand, offset, &len, _rw_page_buf) == 0) {
			/* MACアドレス値が書き込まれていたら、そのMACアドレス値を返す */
			if (_rw_page_buf[ 0] != '\0'
			 && _rw_page_buf[ 2] == ':' && _rw_page_buf[ 5] == ':' && _rw_page_buf[ 8] == ':'
			 && _rw_page_buf[11] == ':' && _rw_page_buf[14] == ':' && _rw_page_buf[17] == '\0') {
				return (char *)_rw_page_buf;
			}
		}
	}
#endif /* CONFIG_CMD_NAND */

	return NULL;
}
#endif /* Add by hiroshi.hasegawa@pixela.jp 2012.04.05 refs #39 @ vanhalen */

