/*
 * EHCI Driver.
 *
 * Copyright (C) 2010 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include <config.h>
#include <common.h>
#include <pci.h>

#include <asm/arch-emma/emma.h>
#include "ehci.h"
#include "ehci-core.h"

#define FLIP_ENDIAN_SWAP	(0x00000002UL)

/*
 * Create the appropriate control structures to manage
 * a new EHCI host controller.
 */
int ehci_hcd_init(void)
{
	u32 d;

	emma_writel(0x00fc0900, FLIP_USB_CTR);
	d = emma_readl(FLIP_M0_CTR);
	d |= FLIP_ENDIAN_SWAP;
	emma_writel(d, FLIP_M0_CTR);

#if 0
	d = emma_readl(FLIP_S_CTR);
	d |= 0x01;
	emma_writel(d, FLIP_S_CTR);
#endif

	d = emma_readl(FLIP_USB_PREFETCH);
	d &= ~0x01;
	emma_writel(d, FLIP_USB_PREFETCH);

	/* PCI BAR */
	emma_writel(CPHYSADDR(EHCI_OPBASE),
			EHCI_CONFBASE + PCI_BASE_ADDRESS_0);

	/* PCI interrupt line */
	d = emma_readl(EHCI_CONFBASE + PCI_INTERRUPT_LINE);
	d &= ~0xff;
	d |= 0x2b;
	emma_writel(d, EHCI_CONFBASE + PCI_INTERRUPT_LINE);

	/* PCI command status */
	emma_writel(PCI_COMMAND_MEMORY | PCI_COMMAND_MASTER,
			EHCI_CONFBASE + PCI_COMMAND);

	hccr = (struct ehci_hccr *)(EHCI_OPBASE);
	hcor = (struct ehci_hcor *)((uint32_t) hccr +
		HC_LENGTH(ehci_readl(&hccr->cr_capbase)));

	return 0;
}

/*
 * Destroy the appropriate control structures corresponding
 * the the EHCI host controller.
 */
int ehci_hcd_stop(void)
{
	return 0;
}

/* Pixela製AitTunerRec基板向け */
#if defined(PIX_AIRTUNER_R) ||  defined(PIX_AIRTUNER_W)
// WLAN, USB hub に電源供給する関数
void ehci_hcd_bus_power_on(void)
{
	u32 d;

	/* WLANの電源供給開始 ＆ USB Hubのリセット解除 をする */
	d = emma_readl(ZPIO_DATA0);
	d |= 0x30LU << 19;
	emma_writel(d, ZPIO_DATA0);

	// USBのHub経由のUSBポートらの電源供給開始前に10ms程間を空ける
#if 1 /* 実際にはmsleep()を使用してCPU使用率をくわないようにしたいが、msleep()が対応していないためusleep()を使用しています */
	udelay(10000);
#else
	msleep(10);
#endif
	
#if defined(PRODUCT_FWIDS)
#if defined(PIX_AIRTUNER_R)
	/* USBのHub経由のUSBポートらの電源供給 強制オフを解除 */
	d = emma_readl(ZPIO_DATA1);
	d &= ~(0x03LU << (47 - 32));
	emma_writel(d, ZPIO_DATA1);
	udelay(1000);
#endif /* defined(PIX_AIRTUNER_R) */
#endif /* defined(PRODUCT_FWIDS) */
	
	/* USBのHub経由のUSBポートらの電源供給開始 */
	d = emma_readl(ZPIO_DATA1);
	d |= 0x03LU << (36 - 32);
	emma_writel(d, ZPIO_DATA1);
}
#endif /* defined(PIX_AIRTUNER_R) ||  defined(PIX_AIRTUNER_W) */
#if defined(PIX_AIRTUNER_LITE)
void ehci_hcd_bus_power_on(void)
{
	u32 d;

	/* USBポートの電源供給を開始する */
	d = emma_readl(ZPIO_DATA1);
	d |= 0x01LU << (36 - 32);
	emma_writel(d, ZPIO_DATA1);
}
#endif /* defined(PIX_AIRTUNER_LITE) */

