/*
 * Copyright (C) 2011 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * This file contains the configuration parameters for EMMA3SE/P board.
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <asm/addrspace.h>
#include <asm/arch-emma/emma3sep.h>
#include <asm/arch-emma/emmaregs.h>
#include <nand_pd.h>
#include <nec_candy_pd.h>

#define CONFIG_BOOT_RETRY_TIME		-1

#define CONFIG_MIPS32			1

#ifndef CPU_CLOCK_RATE
#define CPU_CLOCK_RATE			400000000
#endif

#define CONFIG_EMMA_DDR3		1

#define CONFIG_CONS_INDEX		1
	#if 1 /* Mod by hiroshi.hasegawa@pixela.jp 2012.02.20 */
/* u-boot起動後すぐにLinux起動 */
#define CONFIG_BOOTDELAY		0
	#else
/* autoboot after 5 seconds */
#define CONFIG_BOOTDELAY		5
	#endif
#define CONFIG_ZERO_BOOTDELAY_CHECK

#define CONFIG_BAUDRATE			115200

#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_TIMESTAMP		/* Print image info with timestamp */

/* add command line history */
#define CONFIG_CMDLINE_EDITING		1

	#if 1 /* Mod by hiroshi.hasegawa@pixela.jp 2012.02.20, 2012.03.09, 2012.03.14 */
/* NAND Flashの容量によってパーティション構成が異なるため、環境変数にある
 * NAND Flash上のLinuxカーネルイメージ先頭の格納アドレス値を容量によって
 * 変更する必要があるため、各容量における格納アドレスを引数とするnboot
 * コマンドの文字列を下記のようにdefineしておき、容量によってデフォルトの
 * 環境変数のnbootコマンドを common/env_nand.c 内で書き換える。
 * (Linuxカーネルイメージ先頭の格納アドレスは trunk/kernel/linux-2.6.27.39
 *  /drivers/mtd/nand/nand_emma_ptt_*.h を参照してください) */
#define PIX_ENV_NBOOT_CMD_FOR_NAND_512MB	"nboot 82000000 0 380000" /* ←※文字数を必ず同じに */
#define PIX_ENV_NBOOT_CMD_FOR_NAND_1GB		"nboot 82000000 0 600000" /* ←  しておくこと！     */
#define PIX_ENV_NBOOT_CMD_DEFAULT			PIX_ENV_NBOOT_CMD_FOR_NAND_512MB
#define PIX_ENV_ETHADDR_DEFAULT				"00:00:4c:80:92:a2"
#if defined(PRODUCT_FWIDS) && ( defined(PIX_AIRTUNER_R) || defined(PIX_AIRTUNER_LITE) || defined(PIX_AIRTUNER_W) )
#define PIX_ENV_CONSOLE						"console=ttyS1,115200"
#else
#define PIX_ENV_CONSOLE						"console=ttyS0,115200"
#endif
#if defined(PRODUCT_FWIDS) && defined(PIX_AIRTUNER_R)
#define PIX_ENV_PRODUCT_FWIDS				" PRODUCT_FWIDS=PIX_AIRTUNER_R"
#elif defined(PRODUCT_FWIDS) && defined(PIX_AIRTUNER_LITE)
#define PIX_ENV_PRODUCT_FWIDS				" PRODUCT_FWIDS=PIX_AIRTUNER_LITE"
#elif defined(PRODUCT_FWIDS) && defined(PIX_AIRTUNER_W)
#define PIX_ENV_PRODUCT_FWIDS				" PRODUCT_FWIDS=PIX_AIRTUNER_W"
#else
#define PIX_ENV_PRODUCT_FWIDS				""
#endif
/* defaultの環境変数でROMブートするよう＆NFSブートへの切り替えが
 * # setenv bootcmd run nfs
 * # saveenv
 * とするだけで(容易に)切り替えが出来るようにようにしておく */
/*
 * [シリアルポートの抑制:refs #226]
 *   LINUX起動後のシリアルポート有効化を
 *    # setenv bootcmd run rom_dev
 *    # saveenv
 *   又は、
 *    # setenv bootcmd run nfs_dev
 *    # saveenv
 *   で切り換えられるようにする。
 */
#define CONFIG_EXTRA_ENV_SETTINGS	"bootlinux=set autostart yes; " PIX_ENV_NBOOT_CMD_DEFAULT "\0" \
                                 	"rom=set bootargs rootfstype=squashfs root=/dev/mtdblock3 ro ip=none " PIX_ENV_PRODUCT_FWIDS "; run bootlinux" "\0" \
                                 	"nfs=set bootargs root=/dev/nfs rw ip=:::::eth0:dhcp" PIX_ENV_PRODUCT_FWIDS "; run bootlinux" "\0" \
                                 	"rom_dev=set bootargs " PIX_ENV_CONSOLE " rootfstype=squashfs root=/dev/mtdblock3 ro ip=none " PIX_ENV_PRODUCT_FWIDS "; run bootlinux" "\0" \
                                 	"nfs_dev=set bootargs " PIX_ENV_CONSOLE " root=/dev/nfs rw ip=:::::eth0:dhcp" PIX_ENV_PRODUCT_FWIDS "; run bootlinux" "\0" \
                                 	"bootcmd=run rom" "\0" \
                                 	"ethaddr=" PIX_ENV_ETHADDR_DEFAULT "\0"
	#else
#define CONFIG_BOOTARGS			""

#define CONFIG_EXTRA_ENV_SETTINGS	""

#define CONFIG_BOOTCOMMAND		""
	#endif

#if defined(CONFIG_NAND_BOOT) || defined(CONFIG_SERIAL_BOOT)
#define CONFIG_SYS_NO_FLASH
#endif

/*
 * Command line configuration.
*/
#include <config_cmd_default.h>

#ifndef CONFIG_SMALL
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_MII
#define CONFIG_CMD_PING
#define CONFIG_CMD_NET
#define CONFIG_CMD_USB
#define CONFIG_CMD_FAT
#define CONFIG_CMD_I2C
#define CONFIG_CMD_EEPROM
#define CONFIG_CMD_SPI
#define CONFIG_CMD_SF
#else
#undef CONFIG_CMD_NET
#endif

#if defined(CONFIG_NAND_BOOT) && !defined(CONFIG_SERIAL_BOOT)
#define CONFIG_CMD_NAND
#define CONFIG_YAFFS2 /* Add for NAND Flashのyaffsパーティション上のファイルを読み書き出来るようにする by hiroshi.hasegawa@pixela.jp 2012.03.19 refs #41 @ vanhalen */
#endif
#define CONFIG_CMD_REGINFO
#define CONFIG_CMD_BSP
#define CONFIG_CMD_ELF
#define CONFIG_CMD_UNZIP

/*
 * Miscellaneous configurable options
 */
/* undef to save memory */
#define CONFIG_SYS_LONGHELP
/* Monitor Command Prompt */
#define CONFIG_SYS_PROMPT		"# "
/* Console I/O Buffer Size */
	#if 1  /* Mod by hiroshi.hasegawa@pixela.jp 2012.03.30 refs #41 @ vanhalen */
	/* FWアップデート起動用の環境変数を実行する際、コマンドをセットした変数が長い
	 * (256byteを超える)と実行できないので、バッファサイズを大きく(2048byteに)する */
#define CONFIG_SYS_CBSIZE		2048
	#else
#define CONFIG_SYS_CBSIZE		256
	#endif
/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE+sizeof(CONFIG_SYS_PROMPT)+16)
/* max number of command args*/
#define CONFIG_SYS_MAXARGS		16

#define CONFIG_SYS_MALLOC_LEN		(512 << 10)

#define CONFIG_SYS_BOOTPARAMS_LEN	(128 << 10)

#define CONFIG_SYS_HZ			1000
#define CONFIG_SYS_MIPS_TIMER_FREQ	(CPU_CLOCK_RATE/2)

#define CONFIG_SYS_SDRAM_BASE		0x80000000

	#if 1  /* Mod by hiroshi.hasegawa@pixela.jp 2012.03.14 refs #18 */
/* AirTunerRecのDDR容量が128Mbyteなので、u-bootが使用するメモリ領域は 0x80000000～ 128Mbyte にしておく
 * (ただし、このdefineは起動時のDDR容量のログ出力にしか使われていないようだが...) */
#define CONFIG_SYS_SDRAM_SIZE		(128 << 20)
	#else
#define CONFIG_SYS_SDRAM_SIZE		(256 << 20)
	#endif

/* default load address */
#define CONFIG_SYS_LOAD_ADDR		0x82000000

#define CONFIG_SYS_MEMTEST_START	0x80000000
#define CONFIG_SYS_MEMTEST_END		0x80800000

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#if !defined(CONFIG_SYS_NO_FLASH)

/* derived from include/configs/dbau1x00.h
 * CFG_MAX_FLASH_SECT is need by include/flash.h
 */
#define CONFIG_SYS_FLASH_16BIT			/* 16-bit wide flash memory */
/* max number of memory banks */
#define CONFIG_SYS_MAX_FLASH_BANKS	1
/* max number of sectors on one chip */
#define CONFIG_SYS_MAX_FLASH_SECT	256

#define PHYS_FLASH_1			0xbf800000 /* Flash Bank #1 */
#define CONFIG_SYS_FLASH_BANKS_LIST	{ PHYS_FLASH_1}
/* We boot from this flash, selected with dip switch */
#define CONFIG_SYS_FLASH_BASE		PHYS_FLASH_1

#define CONFIG_SYS_FLASH_PROTECTION	1

/* timeout values are in ticks */
/* Timeout for Flash Erase */
#define CONFIG_SYS_FLASH_ERASE_TOUT	(2 * CONFIG_SYS_HZ)
/* Timeout for Flash Write */
#define CONFIG_SYS_FLASH_WRITE_TOUT	(2 * CONFIG_SYS_HZ)

#define CONFIG_FLASH_CFI_DRIVER
#undef CONFIG_SYS_WRITE_SWAPPED_DATA
#define CONFIG_SYS_FLASH_CFI
#define CONFIG_SYS_FLASH_CFI_WIDTH	FLASH_CFI_16BIT
/* print 'E' for empty sector on flinfo */
#define CONFIG_SYS_FLASH_EMPTY_INFO

#define CONFIG_ENV_IS_IN_FLASH
#define CONFIG_ENV_OVERWRITE

/* Address and size of Linux kernel on FlashROM */
#define CONFIG_SYS_KERNEL_ADDR		0xBF800000
#define CONFIG_SYS_KERNEL_SIZE		(4 << 20)

/* Address and size of Primary Environment Sector	*/
#define CONFIG_ENV_ADDR			0xBFE00000
#define CONFIG_ENV_SIZE			(64 << 10)
#define CONFIG_ENV_SECT_SIZE		(128 << 10)
#define CONFIG_FLASH_16BIT
#undef CONFIG_SYS_DIRECT_FLASH_TFTP

#else
#define CONFIG_SYS_FLASH_BASE		0xbf800000
#endif /* !CONFIG_SYS_NO_FLASH */

/* The following #defines are needed to get flash environment right */
#define CONFIG_SYS_MONITOR_BASE		TEXT_BASE
#define CONFIG_SYS_MONITOR_LEN		(1024 << 10)

#define CONFIG_SYS_INIT_SP_OFFSET	0x400000

#define CONFIG_MISC_INIT_R		/* call misc_init_r() on init	*/


/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CONFIG_SYS_DCACHE_SIZE		(16 << 10) /* 16kB */
#define CONFIG_SYS_ICACHE_SIZE		(16 << 10) /* 16kB */
#define CONFIG_SYS_CACHELINE_SIZE	16

/*-----------------------------------------------------------------------
 * Serial Configuration
 */
#define CONFIG_SYS_NS16550
#define CONFIG_SYS_NS16550_SERIAL
#define CONFIG_SYS_NS16550_REG_SIZE	16
#define CONFIG_SYS_NS16550_CLK		18432000
#if defined(PRODUCT_FWIDS)
#if defined(PIX_AIRTUNER_R) || defined(PIX_AIRTUNER_LITE) || defined(PIX_AIRTUNER_W)
	/* Pixela製AitTunerRec基板 および NEC/東芝向け*/
#define CONFIG_SYS_NS16550_COM1		0xb0102000
#elif defined(PIX_XT046)
	/* Pixela製XT046(コアボード)基板 */
#define CONFIG_SYS_NS16550_COM1		0xb0101000
#elif defined(RENESAS_REF_BRD)
	/* Renesas製リファレンスボード */
#define CONFIG_SYS_NS16550_COM1		0xb0101000
#else
	#error unknown PRODUCT_FWIDS
#endif
#else  /* defined(PRODUCT_FWIDS) */
#define CONFIG_SYS_NS16550_COM1		0xb0101000
#endif /* defined(PRODUCT_FWIDS) */

/*-----------------------------------------------------------------------
 * Ethernet Configuration
 */
#ifdef CONFIG_CMD_NET
#define CONFIG_NET_MULTI
#define CONFIG_NEC_CANDY	/* Candy driver */
#define CONFIG_MII
#define CONFIG_SYS_RX_ETH_BUFFER	4
#define ETH0_BASE			KSEG1ADDR(ETHER_BASE)
#define ETH0_PHY_ADDR			0x00
#define MIIC_CLK			MIIC_CLKS100
#endif

/*-----------------------------------------------------------------------
 * NAND Configuration
 */
#ifdef CONFIG_CMD_NAND
#define CONFIG_NAND_EMMA
#define CONFIG_SYS_MAX_NAND_DEVICE		1
#define CONFIG_SYS_NAND_BASE			0xbf800000

#ifdef CONFIG_NAND_BOOT
#define CONFIG_ENV_IS_IN_NAND
#define CONFIG_ENV_SIZE				(128 << 10)
	#if 1  /* Mod by hiroshi.hasegawa@pixela.jp 2012.03.26 */
/* 不良ブロックがあっても大丈夫なようにしておく(不良ブロックが3つあっても大丈夫にしておく) */
#define CONFIG_ENV_RANGE			(CONFIG_ENV_SIZE * 4)
/* NAND Flash上の環境変数保存領域を、NAND Flash先頭のu-boot用のパーティション終端に変更
 * (NAND Flash先頭のu-boot用のパーティションのサイズは 512Mbit(64Mbyte)のNAND Flashでは
 *  0x200000byte(2.0Mbyte)、1Gbit(128Mbyte)のNAND Flashでは0x300000byte(3.0Mbyte)なので
 *  環境変数保存領域は、0x200000番地より前の CONFIG_ENV_RANGE byteにしておく)
 * また ミラーリングは不要な(製品では環境変数はNAND Flash上に保存されず、ソフトウェアに
 * ハードコーディングされたdefaultの環境変数が使用される)ので CONFIG_ENV_OFFSET_REDUND は
 * defineしない！*/
#define CONFIG_ENV_OFFSET			(0x200000 - CONFIG_ENV_RANGE)
/* FWアップデート起動用の環境変数(NAND Flashの書き換え処理を組み込んだ)の保存領域 refs #41 @ vanhalen
 * (mtd1パーティションの先頭アドレスです。mtdパーティションの先頭アドレスは
 *  trunk/kernel/linux-2.6.27.39/drivers/mtd/nand/nand_emma_ptt_*.h を参照してください) */
#define CONFIG_ENV_UPDATE_OFFSET_512MB	0x200000
#define CONFIG_ENV_UPDATE_OFFSET_1GB	0x300000
/* MACアドレス値の保存領域 refs #39 @ vanhalen
 * (mtd1パーティションの最終Blockの先頭アドレスです。
 *  trunk/kernel/linux-2.6.27.39/drivers/mtd/nand/nand_emma_ptt_*.h を参照してください) */
#define PIX_ENV_ETHADDR_OFFSET_FOR_NAND_512MB(blocksize)	(0x380000 - blocksize)
#define PIX_ENV_ETHADDR_OFFSET_FOR_NAND_1GB(blocksize)		(0x600000 - blocksize)
	#else  /* Mod by hiroshi.hasegawa@pixela.jp 2012.03.26 */
#define CONFIG_ENV_OFFSET			0x00700000
#define CONFIG_ENV_OFFSET_REDUND		0x00720000
	#endif /* Mod by hiroshi.hasegawa@pixela.jp 2012.03.26 */
#define CONFIG_ENV_OVERWRITE
#endif

#define CONFIG_SYS_NAND_SMALL_PAGE		1
#define CONFIG_SYS_NAND_CACHEPRG_SUPPORT	0
#define CONFIG_SYS_NAND_CS			NAND_CS0
#endif

/*-----------------------------------------------------------------------
 * USB Configuration
 */
#ifdef CONFIG_CMD_USB
/* OHCI Configuration */
#if 0
#define CONFIG_USB_OHCI_EMMA
#define CONFIG_USB_OHCI_NEW
#define CONFIG_SYS_USB_OHCI_BOARD_INIT		1
#define	CONFIG_SYS_USB_OHCI_MAX_ROOT_PORTS	2
#define CONFIG_SYS_USB_OHCI_REGS_BASE		0xb0201000
#define CONFIG_SYS_USB_OHCI_SLOT_NAME		"emma-ohci"
#endif

#if 1
/* EHCI Configuration */
#define CONFIG_USB_EHCI
#define CONFIG_USB_EHCI_EMMA
#define CONFIG_EHCI_DCACHE
/*#define CONFIG_EHCI_DESC_BIG_ENDIAN*/
#define CONFIG_EHCI_MMIO_BIG_ENDIAN
#define CONFIG_SYS_USB_EHCI_MAX_ROOT_PORTS	2
#endif

/* USB Device Configuration */
#define CONFIG_USB_STORAGE
#define CONFIG_USB_DEVICE
#define CONFIG_DOS_PARTITION
#endif

/*-----------------------------------------------------------------------
 * I2C Configuration
 */
#ifdef CONFIG_CMD_I2C
#define CONFIG_EMMA_I2C
#define CONFIG_HARD_I2C
#define CONFIG_SYS_I2C_SPEED			400000
#define CONFIG_SYS_I2C_SLAVE			0
#define CONFIG_SYS_I2C_INIT_BOARD
#define CONFIG_I2C_MULTI_BUS
#define CONFIG_SYS_MAX_I2C_BUS			3
#define CONFIG_SYS_EEPROM_PAGE_WRITE_DELAY_MS	5

#define CONFIG_SYS_I2C_EEPROM_ADDR		0x50
#define CONFIG_SYS_I2C_EEPROM_ADDR_LEN		2
#endif

/*-----------------------------------------------------------------------
 * Serial Flash Configuration
 */
#ifdef CONFIG_CMD_SPI
#define CONFIG_EMMA_SPI
#define CONFIG_SPI_FLASH
#define CONFIG_SPI_FLASH_WINBOND
#define CONFIG_SPI_FLASH_STMICRO

#ifdef CONFIG_SERIAL_BOOT
#define CONFIG_ENV_IS_IN_SPI_FLASH
#define CONFIG_ENV_SIZE				(4 << 10)
#define CONFIG_ENV_SECT_SIZE			(64 << 10)
#define CONFIG_ENV_OFFSET			0x003f0000
#define CONFIG_ENV_OVERWRITE
#endif
#endif

#endif	/* __CONFIG_H */

