/*
 * Copyright (C) 2011 Renesas Electronics Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 * This file contains the configuration parameters for EMMA3SLx board.
 */

#ifndef __CONFIG_H
#define __CONFIG_H

#include <asm/addrspace.h>
#include <asm/arch-emma/emma3slp.h>
#include <asm/arch-emma/emmaregs.h>
#include <nand_pd.h>
#include <nec_candy_pd.h>

#define CONFIG_BOOT_RETRY_TIME		-1

#define CONFIG_MIPS32			1

#ifndef CPU_CLOCK_RATE
#define CPU_CLOCK_RATE			324000000
#endif

#define CONFIG_CONS_INDEX		1
/* autoboot after 5 seconds */
#define CONFIG_BOOTDELAY		5
#define CONFIG_ZERO_BOOTDELAY_CHECK

#define CONFIG_BAUDRATE			115200

#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_TIMESTAMP		/* Print image info with timestamp */

/* add command line history */
#define CONFIG_CMDLINE_EDITING		1

#define CONFIG_BOOTARGS			""

#define CONFIG_EXTRA_ENV_SETTINGS	""

#define CONFIG_BOOTCOMMAND		""

#if defined(CONFIG_NAND_BOOT) || defined(CONFIG_SERIAL_BOOT)
#define CONFIG_SYS_NO_FLASH
#endif

/*
 * Command line configuration.
*/
#include <config_cmd_default.h>

#ifndef CONFIG_SMALL
#define CONFIG_CMD_DHCP
#define CONFIG_CMD_MII
#define CONFIG_CMD_PING
#define CONFIG_CMD_NET
#define CONFIG_CMD_USB
#define CONFIG_CMD_FAT
#define CONFIG_CMD_I2C
#define CONFIG_CMD_EEPROM
#define CONFIG_CMD_SPI
#define CONFIG_CMD_SF
#else
#undef CONFIG_CMD_NET
#endif

#if defined(CONFIG_NAND_BOOT) && !defined(CONFIG_SERIAL_BOOT)
#define CONFIG_CMD_NAND
#endif
#define CONFIG_CMD_REGINFO
#define CONFIG_CMD_BSP
#define CONFIG_CMD_ELF
#define CONFIG_CMD_UNZIP

/*
 * Miscellaneous configurable options
 */
/* undef to save memory */
#define CONFIG_SYS_LONGHELP
/* Monitor Command Prompt */
#define CONFIG_SYS_PROMPT		"# "
/* Console I/O Buffer Size */
#define CONFIG_SYS_CBSIZE		256
/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE+sizeof(CONFIG_SYS_PROMPT)+16)
/* max number of command args*/
#define CONFIG_SYS_MAXARGS		16

#define CONFIG_SYS_MALLOC_LEN		(512 << 10)

#define CONFIG_SYS_BOOTPARAMS_LEN	(128 << 10)

#define CONFIG_SYS_HZ			1000
#define CONFIG_SYS_MIPS_TIMER_FREQ	(CPU_CLOCK_RATE/2)

#define CONFIG_SYS_SDRAM_BASE		0x80000000

#define CONFIG_SYS_SDRAM_SIZE		(256 << 20)

/* default load address */
#define CONFIG_SYS_LOAD_ADDR		0x82000000

#define CONFIG_SYS_MEMTEST_START	0x80000000
#define CONFIG_SYS_MEMTEST_END		0x80800000

/*-----------------------------------------------------------------------
 * FLASH and environment organization
 */
#if !defined(CONFIG_SYS_NO_FLASH)

/* derived from include/configs/dbau1x00.h
 * CFG_MAX_FLASH_SECT is need by include/flash.h
 */
#define CONFIG_SYS_FLASH_16BIT			/* 16-bit wide flash memory */
/* max number of memory banks */
#define CONFIG_SYS_MAX_FLASH_BANKS	1
/* max number of sectors on one chip */
#define CONFIG_SYS_MAX_FLASH_SECT	512

#define PHYS_FLASH_1			0xbf800000 /* Flash Bank #1 */
#define CONFIG_SYS_FLASH_BANKS_LIST	{ PHYS_FLASH_1}
/* We boot from this flash, selected with dip switch */
#define CONFIG_SYS_FLASH_BASE		PHYS_FLASH_1

#define CONFIG_SYS_FLASH_PROTECTION	1

/* timeout values are in ticks */
/* Timeout for Flash Erase */
#define CONFIG_SYS_FLASH_ERASE_TOUT	(2 * CONFIG_SYS_HZ)
/* Timeout for Flash Write */
#define CONFIG_SYS_FLASH_WRITE_TOUT	(2 * CONFIG_SYS_HZ)

#define CONFIG_FLASH_CFI_DRIVER
#undef CONFIG_SYS_WRITE_SWAPPED_DATA
#define CONFIG_SYS_FLASH_CFI
#define CONFIG_SYS_FLASH_CFI_WIDTH	FLASH_CFI_16BIT
/* print 'E' for empty sector on flinfo */
#define CONFIG_SYS_FLASH_EMPTY_INFO

#define CONFIG_ENV_IS_IN_FLASH
#define CONFIG_ENV_OVERWRITE

/* Address and size of Linux kernel on FlashROM */
#define CONFIG_SYS_KERNEL_ADDR		0xBF800000
#define CONFIG_SYS_KERNEL_SIZE		(4 << 20)

/* Address and size of Primary Environment Sector	*/
#define CONFIG_ENV_ADDR			0xBFE00000
#define CONFIG_ENV_SIZE			(64 << 10)
#define CONFIG_ENV_SECT_SIZE		(128 << 10)
#define CONFIG_FLASH_16BIT
#undef CONFIG_SYS_DIRECT_FLASH_TFTP

#else
#define CONFIG_SYS_FLASH_BASE		0xbf000000
#endif /* !CONFIG_SYS_NO_FLASH */

/* The following #defines are needed to get flash environment right */
#define CONFIG_SYS_MONITOR_BASE		TEXT_BASE
#define CONFIG_SYS_MONITOR_LEN		(1024 << 10)

#define CONFIG_SYS_INIT_SP_OFFSET	0x400000

#define CONFIG_MISC_INIT_R		/* call misc_init_r() on init	*/


/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CONFIG_SYS_DCACHE_SIZE		(16 << 10) /* 16kB */
#define CONFIG_SYS_ICACHE_SIZE		(16 << 10) /* 16kB */
#define CONFIG_SYS_CACHELINE_SIZE	16

/*-----------------------------------------------------------------------
 * Serial Configuration
 */
#define CONFIG_SYS_NS16550
#define CONFIG_SYS_NS16550_SERIAL
#define CONFIG_SYS_NS16550_REG_SIZE	16
#define CONFIG_SYS_NS16550_CLK		18432000
#define CONFIG_SYS_NS16550_COM1		0xb0101000

/*-----------------------------------------------------------------------
 * Ethernet Configuration
 */
#ifdef CONFIG_CMD_NET
#define CONFIG_NET_MULTI
#define CONFIG_NEC_CANDY	/* Candy driver */
#define CONFIG_MII
#define CONFIG_SYS_RX_ETH_BUFFER	4
#define ETH0_BASE			KSEG1ADDR(ETHER_BASE)
#define ETH0_PHY_ADDR			0x00
#define MIIC_CLK			MIIC_CLKS100
#endif

/*-----------------------------------------------------------------------
 * NAND Configuration
 */
#ifdef CONFIG_CMD_NAND
#define CONFIG_NAND_EMMA
#define CONFIG_SYS_MAX_NAND_DEVICE		1
#define CONFIG_SYS_NAND_BASE			0xbf000000

#ifdef CONFIG_NAND_BOOT
#define CONFIG_ENV_IS_IN_NAND
#define CONFIG_ENV_SIZE				(128 << 10)
#define CONFIG_ENV_OFFSET			0x00700000
#define CONFIG_ENV_OFFSET_REDUND		0x00720000
#define CONFIG_ENV_OVERWRITE
#endif

#define CONFIG_SYS_NAND_LARGE_PAGE		1
#define CONFIG_SYS_NAND_CACHEPRG_SUPPORT	0
#define CONFIG_SYS_NAND_CS			NAND_CS1
#endif

/*-----------------------------------------------------------------------
 * USB Configuration
 */
#ifdef CONFIG_CMD_USB
/* OHCI Configuration */
#if 0
#define CONFIG_USB_OHCI_EMMA
#define CONFIG_USB_OHCI_NEW
#define CONFIG_SYS_USB_OHCI_BOARD_INIT		1
#define	CONFIG_SYS_USB_OHCI_MAX_ROOT_PORTS	2
#define CONFIG_SYS_USB_OHCI_REGS_BASE		0xb0201000
#define CONFIG_SYS_USB_OHCI_SLOT_NAME		"emma-ohci"
#endif

#if 1
/* EHCI Configuration */
#define CONFIG_USB_EHCI
#define CONFIG_USB_EHCI_EMMA
#define CONFIG_EHCI_DCACHE
/*#define CONFIG_EHCI_DESC_BIG_ENDIAN*/
#define CONFIG_EHCI_MMIO_BIG_ENDIAN
#define CONFIG_SYS_USB_EHCI_MAX_ROOT_PORTS	2
#endif

/* USB Device Configuration */
#define CONFIG_USB_STORAGE
#define CONFIG_USB_DEVICE
#define CONFIG_DOS_PARTITION
#endif

/*-----------------------------------------------------------------------
 * I2C Configuration
 */
#ifdef CONFIG_CMD_I2C
#define CONFIG_EMMA_I2C
#define CONFIG_HARD_I2C
#define CONFIG_SYS_I2C_SPEED			400000
#define CONFIG_SYS_I2C_SLAVE			0
#define CONFIG_SYS_I2C_INIT_BOARD
#define CONFIG_I2C_MULTI_BUS
#define CONFIG_SYS_MAX_I2C_BUS			2
#define CONFIG_SYS_EEPROM_PAGE_WRITE_DELAY_MS	5

#define CONFIG_SYS_I2C_EEPROM_ADDR		0x50
#define CONFIG_SYS_I2C_EEPROM_ADDR_LEN		2
#endif

/*-----------------------------------------------------------------------
 * Serial Flash Configuration
 */
#ifdef CONFIG_CMD_SPI
#define CONFIG_EMMA_SPI
#define CONFIG_SPI_FLASH
#define CONFIG_SPI_FLASH_WINBOND
#define CONFIG_SPI_FLASH_STMICRO

#ifdef CONFIG_SERIAL_BOOT
#define CONFIG_ENV_IS_IN_SPI_FLASH
#define CONFIG_ENV_SIZE				(4 << 10)
#define CONFIG_ENV_SECT_SIZE			(4 << 10)
#define CONFIG_ENV_OFFSET			0x003fe000
#define CONFIG_ENV_OVERWRITE
#endif
#endif

#endif	/* __CONFIG_H */

